#!/usr/bin/env python
import user
import importer
import unittest

from lex import LexToken

class CxxPreprocessor (unittest.TestCase):
  '''Tests the Cxx preprocessor implementation'''
  def setUp(self):
    import ASE.Compiler.Cxx_impl.cIncludeExpansion

    self.expander = ASE.Compiler.Cxx_impl.cIncludeExpansion.IncludeExpander()
    return

  def tearDown(self):
    return

  def testConditionalExpressions(self):
    '''Verify that expressions in #if conditionals function correctly'''
    code = '''\
    #if ! 0
      correct
    #else
      error
    #endif
    '''
    self.assertEquals(self.expander.process(code), [LexToken('ID','correct',2)])
    code = '''\
    #if 0 || 1
      correct
    #else
      error
    #endif
    '''
    self.assertEquals(self.expander.process(code), [LexToken('ID','correct',2)])
    code = '''\
    #if 0 || 1 || 2
      correct
    #else
      error
    #endif
    '''
    self.assertEquals(self.expander.process(code), [LexToken('ID','correct',2)])
    code = '''\
    #if 0 && ! 0
      error
    #else
      correct
    #endif
    '''
    self.assertEquals(self.expander.process(code), [LexToken('ID','correct',4)])
    return

  def testNestedDefine(self):
    '''Verify that a #define nested inside an #if block will take effect'''
    defineTokens = [LexToken('POUND','#',2), LexToken('ID','define',2), LexToken('ID','__MATT__',2)]
    code = '''\
    #if 1
      #define __MATT__
    #endif
    #ifdef __MATT__
      correct
    #endif'''
    self.assertEquals(self.expander.process(code), defineTokens+[LexToken('ID','correct',5)])
    return

  def testIntegerDefine(self):
    '''Verify that a simple define to an integer is expanded correctly and behaves correctly in conditionals'''
    defineTokens = [LexToken('POUND','#',1), LexToken('ID','define',1), LexToken('ID','__MATT__',1), LexToken('ICONST','1',1)]
    code = '''\
    #define __MATT__ 1
    void foo(int i) {
      int j = __MATT__;

      return i + bar(__MATT__) + j;
    }
    '''
    self.assertEquals(self.expander.process(code),
                      defineTokens+[LexToken('VOID','void',2),LexToken('ID','foo',2),LexToken('LPAREN','(',2),LexToken('INT','int',2),LexToken('ID','i',2),
                                    LexToken('RPAREN',')',2),LexToken('LBRACE','{',2),
                                    LexToken('INT','int',3),LexToken('ID','j',3),LexToken('EQUALS','=',3),LexToken('ICONST','1',3),LexToken('SEMI',';',3),
                                    LexToken('RETURN','return',5),LexToken('ID','i',5),LexToken('PLUS','+',5),LexToken('ID','bar',5),LexToken('LPAREN','(',5),
                                    LexToken('ICONST','1',5),LexToken('RPAREN',')',5),LexToken('PLUS','+',5),LexToken('ID','j',5),LexToken('SEMI',';',5),
                                    LexToken('RBRACE','}',6)])
    code = '''\
    #define __MATT__ 1
    #ifdef __MATT__
      correct
    #endif
    #if __MATT__
      correct
    #endif
    #define __MATT__ 0
    #if __MATT__
      error
    #endif'''
    self.assertEquals(self.expander.process(code),
                      defineTokens+[LexToken('ID','correct',3),LexToken('ID','correct',6),
                                    LexToken('POUND','#',8), LexToken('ID','define',8), LexToken('ID','__MATT__',8), LexToken('ICONST','0',8)])
    return

  def testEmptyDefine(self):
    '''Verify that an empty define statement triggers an #ifdef and #ifndef'''
    defineTokens = [LexToken('POUND','#',1), LexToken('ID','define',1), LexToken('ID','__MATT__',1)]
    code = '''\
    #define __MATT__
    #ifdef __MATT__
      correct
    #endif'''
    self.assertEquals(self.expander.process(code), defineTokens+[LexToken('ID','correct',3)])
    code = '''\
    #define __MATT__
    #ifdef __MATT__
      correct
    #else
      error
    #endif'''
    self.assertEquals(self.expander.process(code), defineTokens+[LexToken('ID','correct',3)])
    code = '''\
    #define __MATT__
    #ifndef __MATT__
      error
    #endif'''
    self.assertEquals(self.expander.process(code), defineTokens)
    code = '''\
    #define __MATT__
    #ifndef __MATT__
      error
    #else
      correct
    #endif'''
    self.assertEquals(self.expander.process(code), defineTokens+[LexToken('ID','correct',5)])
    return

  def testConstantExpression(self):
    '''Verify that constants are handled correctly'''
    code = '''\
    #if 0
      error
    #endif'''
    self.assertEquals(self.expander.process(code), [])
    code = '''\
    #if 1
      correct
    #endif'''
    self.assertEquals(self.expander.process(code), [LexToken('ID','correct',2)])
    return

  def testComments(self):
    '''Verify that these comments are ignored'''
    code = '''/**/'''
    self.assertEquals(self.expander.process(code), [])
    code = '''/* Matt */'''
    self.assertEquals(self.expander.process(code), [])
    code = '''/* Matt*2 */'''
    self.assertEquals(self.expander.process(code), [])
    code = '''/* Matt*2*3*4*5678**/'''
    self.assertEquals(self.expander.process(code), [])
    code = '''/*
Matt
  likes*
    beer
*/'''
    self.assertEquals(self.expander.process(code), [])
    code = '''/*
Problem

  Python modules written in C that must access Python objects must do
  so through routines whose interfaces are described by a set of
  include files.  Unfortunately, these routines vary according to the
  object accessed.  To use these routines, the C programmer must check
  the type of the object being used and must call a routine based on
  the object type.  For example, to access an element of a sequence,
  the programmer must determine whether the sequence is a list or a
  tuple:

    if(is_tupleobject(o))
      e=gettupleitem(o,i)
    else if(is_listitem(o))
      e=getlistitem(o,i)

  If the programmer wants to get an item from another type of object
  that provides sequence behavior, there is no clear way to do it
  correctly.  

  The persistent programmer may peruse object.h and find that the
  _typeobject structure provides a means of invoking up to (currently
  about) 41 special operators.  So, for example, a routine can get an
  item from any object that provides sequence behavior. However, to
  use this mechanism, the programmer must make their code dependent on
  the current Python implementation.

  Also, certain semantics, especially memory management semantics, may
  differ by the type of object being used.  Unfortunately, these
  semantics are not clearly described in the current include files.
  An abstract interface providing more consistent semantics is needed. 

Proposal

  I propose the creation of a standard interface (with an associated
  library of routines and/or macros) for generically obtaining the
  services of Python objects.  This proposal can be viewed as one
  components of a Python C interface consisting of several components.

  From the viewpoint of C access to Python services, we have (as
  suggested by Guido in off-line discussions):

  - "Very high level layer": two or three functions that let you exec or
    eval arbitrary Python code given as a string in a module whose name is
    given, passing C values in and getting C values out using
    mkvalue/getargs style format strings.  This does not require the user
    to declare any variables of type "PyObject *".  This should be enough
    to write a simple application that gets Python code from the user,
    execs it, and returns the output or errors.  (Error handling must also
    be part of this API.)

  - "Abstract objects layer": which is the subject of this proposal.
    It has many functions operating on objects, and lest you do many
    things from C that you can also write in Python, without going
    through the Python parser.

  - "Concrete objects layer": This is the public type-dependent
    interface provided by the standard built-in types, such as floats,
    strings, and lists.  This interface exists and is currently
    documented by the collection of include files provided with the
    Python distributions.

  From the point of view of Python accessing services provided by C
  modules: 

  - "Python module interface": this interface consist of the basic
    routines used to define modules and their members.  Most of the
    current extensions-writing guide deals with this interface.

  - "Built-in object interface": this is the interface that a new
    built-in type must provide and the mechanisms and rules that a
    developer of a new built-in type must use and follow.

  This proposal is a "first-cut" that is intended to spur
  discussion. See especially the lists of notes.

  The Python C object interface will provide four protocols: object,
  numeric, sequence, and mapping.  Each protocol consists of a
  collection of related operations.  If an operation that is not
  provided by a particular type is invoked, then a standard exception,
  NotImplementedError is raised with a operation name as an argument.
  In addition, for convenience this interface defines a set of
  constructors for building objects of built-in types.  This is needed
  so new objects can be returned from C functions that otherwise treat
  objects generically.

Memory Management

  For all of the functions described in this proposal, if a function
  retains a reference to a Python object passed as an argument, then the
  function will increase the reference count of the object.  It is
  unnecessary for the caller to increase the reference count of an
  argument in anticipation of the objects retention.

  All Python objects returned from functions should be treated as new
  objects.  Functions that return objects assume that the caller will
  retain a reference and the reference count of the object has already
  been incremented to account for this fact.  A caller that does not
  retain a reference to an object that is returned from a function
  must decrement the reference count of the object (using
  DECREF(object)) to prevent memory leaks.

  Note that the behavior mentioned here is different from the current
  behavior for some objects (e.g. lists and tuples) when certain
  type-specific routines are called directly (e.g. setlistitem).  The
  proposed abstraction layer will provide a consistent memory
  management interface, correcting for inconsistent behavior for some
  built-in types.

Protocols

xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/'''
    self.assertEquals(self.expander.process(code), [])
    code = '''\
    /* before */
    #define MIDDLE 1
    /* after */'''
    self.assertEquals(self.expander.process(code), [LexToken('POUND','#',2), LexToken('ID','define',2), LexToken('ID','MIDDLE',2), LexToken('ICONST','1',2)])
    code = '''\
    /* Multi
       line
       before */
    #define MIDDLE 1
    /* after */'''
    self.assertEquals(self.expander.process(code), [LexToken('POUND','#',4), LexToken('ID','define',4), LexToken('ID','MIDDLE',4), LexToken('ICONST','1',4)])
    return

if __name__ == '__main__':
  unittest.main()
