class Language(object):
  
  def isIn(self, parameter):
    import ASE.Compiler.ParameterAccessMode
    mode = parameter.getAccessMode()
    return mode == ASE.Compiler.ParameterAccessMode.IN or mode == ASE.Compiler.ParameterAccessMode.INOUT
  
  def isOut(self, parameter):
    import ASE.Compiler.ParameterAccessMode
    mode = parameter.getAccessMode()
    return mode == ASE.Compiler.ParameterAccessMode.OUT or mode == ASE.Compiler.ParameterAccessMode.INOUT or mode == ASE.Compiler.ParameterAccessMode.RETURN

class Python(Language):
  '''
  This class is intended to encapsulate operations which are generic to the Python language (both Python itself and C extensions)
  '''
  
  def __init__(self):
    import ASE.Compiler.Python.IOR
    self.IOR = ASE.Compiler.Python.IOR.IOR()
    self.Cxx = self.IOR.Cxx
    self.SIDL = self.IOR.SIDL
    self.typeMap = self.createBaseTypeMap()
    self.typeCodes = self.createBaseTypeCodes()
    self.clear()
    return

  def clear(self):
    self.Cxx.clear()
    self.refTypeType = None
    self.objType = None
    self.arrayType = None
    self.typeType = None
    self.sidlObjTypes = {}

    self.wrapIOR = {}

    self.extractIOR = {}

    self.wrapArray = {}

    self.extractArray = {}

    return

  def getString(self, s):
    import ASE.Compiler.Python.Const
    string = ASE.Compiler.Python.Const.Const()
    string.setValue_string('\'' + s + '\'')
    return string
  
  def createBaseTypeMap(self):
    '''
    Create a map from SIDL base types to C extension types
    '''
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    typeMap['bool'] = self.IOR.typeMap['bool']
    typeMap['char'] = self.IOR.typeMap['char']
    pyType = ASE.Compiler.Cxx.Type.Type()
    pyType.setIdentifier('Py_complex')
    typeMap['dcomplex'] = pyType
    typeMap['double'] = self.IOR.typeMap['double']
    typeMap['fcomplex'] = typeMap['dcomplex']
    typeMap['float'] = self.IOR.typeMap['float']
    pyType = ASE.Compiler.Cxx.Type.Type()
    pyType.setIdentifier('long')
    pyType.setBaseType(1)
    typeMap['int'] = pyType
    pyType = ASE.Compiler.Cxx.Type.Type()
    pyType.setIdentifier('long long')
    pyType.setBaseType(1)
    typeMap['long'] = pyType
    typeMap['void'] = self.IOR.typeMap['void']
    typeMap['opaque'] = self.IOR.typeMap['opaque']
    typeMap['string'] = self.IOR.typeMap['string']
    return typeMap
  
  def getType(self, type):
    '''
    Return the Python C extension type corresponding to the given SIDL type
    '''
    if type.getBaseType():
      try:
        pyType = self.typeMap[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      pyType = self.typeMap['int']
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      pyType = self.IOR.getType(type)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      pyType = self.IOR.getType(type)
    else:
      self.raiseException('Unable to map type: ' + str(type.getFullIdentifier()))
    return pyType
  
  def createBaseTypeCodes(self):
    '''
    Create a map from SIDL base types to Numeric typecodes
    '''
    typeCodes = {}

    typeCodes['char'] = self.Cxx.getVar('PyArray_CHAR')
    typeCodes['dcomplex'] = self.Cxx.getVar('PyArray_CDOUBLE')
    typeCodes['double'] = self.Cxx.getVar('PyArray_DOUBLE')
    typeCodes['fcomplex'] = self.Cxx.getVar('PyArray_CFLOAT')
    typeCodes['float'] = self.Cxx.getVar('PyArray_FLOAT')
    typeCodes['int'] = self.Cxx.getVar('PyArray_INT')
    typeCodes['bool'] = typeCodes['int']
    typeCodes['long'] = self.Cxx.getVar('PyArray_LONG')
    typeCodes['object'] = self.Cxx.getVar('PyArray_OBJECT')
    typeCodes['opaque'] = typeCodes['object']
    typeCodes['string'] = typeCodes['object']
    typeCodes['void'] = self.Cxx.getVar('PyArray_NOTYPE')
    return typeCodes
  
  def getNumericTypeCode(self, type):
    '''
    Return the Numeric typecode corresponding to the given SIDL type
    '''
    if type.getBaseType():
      try:
        typeCode = self.typeCodes[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      typeCode = self.typeCodes['int']
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.raiseException('Nested arrays not allowed in SIDL')
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      typeCode = self.typeCodes['object']
    else:
      typeCode = self.typeCodes['void']
    return typeCode
  
  def getRefTypeType(self):
    '''
    Return the Python C extension type used to indicate the type of Python reference
    '''
    if self.refTypeType is None:
      import ASE.Compiler.Cxx.Enumeration
      self.refTypeType = ASE.Compiler.Cxx.Enumeration.Enumeration()
      self.refTypeType.setIdentifier('ReferenceType')
    return self.refTypeType
  
  def getObjectType(self):
    '''
    Return the Python C extension type for Python instance objects
    '''
    if self.objType is None:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Type
      objStruct = ASE.Compiler.Cxx.Type.Type()
      objStruct.setIdentifier('PyObject')
      self.objType = ASE.Compiler.Cxx.Pointer.Pointer()
      self.objType.setChildren([objStruct])
    return self.objType
  
  def getArrayType(self):
    '''
    Return the Python C extension type for Numeric array objects
    '''
    if self.arrayType is None:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Type
      arrayStruct = ASE.Compiler.Cxx.Type.Type()
      arrayStruct.setIdentifier('PyArrayObject')
      self.arrayType = ASE.Compiler.Cxx.Pointer.Pointer()
      self.arrayType.setChildren([arrayStruct])
    return self.arrayType
  
  def getTypeType(self):
    '''
    Return the Python C extension type for Python type objects
    '''
    if self.typeType is None:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Type
      typeStruct = ASE.Compiler.Cxx.Type.Type()
      typeStruct.setIdentifier('PyTypeObject')
      self.typeType = ASE.Compiler.Cxx.Pointer.Pointer()
      self.typeType.setChildren([typeStruct])
    return self.typeType
  
  def getSIDLObjectType(self, interface):
    '''
    Return the specific Python C extension type for Python instance objects for the given SIDL interface
    '''
    if not interface in self.sidlObjTypes:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      objStruct = ASE.Compiler.Cxx.Struct.Struct()
      objStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_PythonObject')
      self.sidlObjTypes[interface] = ASE.Compiler.Cxx.Pointer.Pointer()
      self.sidlObjTypes[interface].setChildren([objStruct])
    return self.sidlObjTypes[interface]
  
  def getRefTypeEnumeration(self):
    '''
    Return the Python C extension enumeration declaration for reference type
    '''
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Enumerator
    stolenRef = ASE.Compiler.Cxx.Enumerator.Enumerator()
    stolenRef.setIdentifier('STOLEN_REF')
    weakRef = ASE.Compiler.Cxx.Enumerator.Enumerator()
    weakRef.setIdentifier('WEAK_REF')
    newRef = ASE.Compiler.Cxx.Enumerator.Enumerator()
    newRef.setIdentifier('NEW_REF')
    refType = ASE.Compiler.Cxx.Enumeration.Enumeration()
    refType.setIdentifier('ReferenceType')
    refType.setChildren([stolenRef, weakRef, newRef])
    refDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    refDecl.setType(refType)
    return self.Cxx.getDecl(refDecl)
  
  def getSIDLObject(self, klass):
    '''
    Return the Python C extension structure declaration for the given class
    '''
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Macro
    import ASE.Compiler.Cxx.Struct
    head = ASE.Compiler.Cxx.Macro.Macro()
    head.setIdentifier('PyObject_HEAD')
    objIOR = ASE.Compiler.Cxx.Declarator.Declarator()
    objIOR.setIdentifier('ior')
    objIOR.setType(self.IOR.getType(klass))
    refType = ASE.Compiler.Cxx.Enumeration.Enumeration()
    refType.setIdentifier('ReferenceType')
    objRef = ASE.Compiler.Cxx.Declarator.Declarator()
    objRef.setIdentifier('refType')
    objRef.setType(refType)
    weakRefList = ASE.Compiler.Cxx.Declarator.Declarator()
    weakRefList.setIdentifier('weakRefList')
    weakRefList.setType(self.getObjectType())
    objStruct = ASE.Compiler.Cxx.Struct.Struct()
    objStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '_PythonObject')
    objStruct.setChildren([self.Cxx.getExpStmt(head), self.Cxx.getDecl(objIOR), self.Cxx.getDecl(objRef), self.Cxx.getDecl(weakRefList)])
    objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    objDecl.setType(objStruct)
    return self.Cxx.getDecl(objDecl)
  
  def getClearError(self):
    '''
    Return the AST which clears any Python error which has been set
       - Should probably be replaced by a PyErr_Fetch()
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    clearError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    clearError.setChildren([self.Cxx.getVar('PyErr_Clear')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(clearError)])
    pythonError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    pythonError.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(pythonError)
    errorCheck.setChildren([compound])
    return errorCheck
  
  def getSuspendError(self):
    '''
    Return the AST which suspends any Python error which has been set
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    suspendError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    suspendError.setChildren([self.Cxx.getVar('PyErr_Fetch')])
    suspendError.setArguments([self.Cxx.getAddress(self.Cxx.getVar('_saveExcType')), self.Cxx.getAddress(self.Cxx.getVar('_saveExcValue')), self.Cxx.getAddress(self.Cxx.getVar('_saveExcTraceback'))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(suspendError)])
    pythonError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    pythonError.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(pythonError)
    errorCheck.setChildren([compound])
    return errorCheck
  
  def getRestoreError(self):
    '''
    Return the AST which restores any Python error which has been set
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    suspendError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    suspendError.setChildren([self.Cxx.getVar('PyErr_Restore')])
    suspendError.setArguments([self.Cxx.getVar('_saveExcType'), self.Cxx.getVar('_saveExcValue'), self.Cxx.getVar('_saveExcTraceback')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(suspendError)])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(self.Cxx.getVar('_saveExcType'))
    errorCheck.setChildren([compound])
    return errorCheck
  
  def getErrorCheck(self, exceptionType, errorMsg, errorVar = None, returnVar = None, branchExp = None):
    '''
    Return the AST for the check for a Python exception
       - exceptionType is the type of exception to be raised on an error, e.g. PyExc_ImportError
       - errorMsg is the string for the raised exception
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Return
    if not errorVar is None:
      import ASE.Compiler.Cxx.Negation
      pythonError = ASE.Compiler.Cxx.Negation.Negation()
      pythonError.setChildren([errorVar])
    elif not branchExp is None:
      pythonError = branchExp
    else:
      pythonError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      pythonError.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    exception = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    exception.setChildren([self.Cxx.getVar('PyErr_SetString')])
    exception.setArguments([self.Cxx.getVar(exceptionType), self.Cxx.getString(errorMsg)])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    if not returnVar is None:
      returnVal.setChildren([returnVar])
    handleError = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    handleError.setChildren([self.Cxx.getExpStmt(exception), returnVal])
    errorCheck = ASE.Compiler.Cxx.If.If()
    errorCheck.setBranch(pythonError)
    errorCheck.setChildren([handleError])
    return errorCheck
  
  def getNew(self, interface):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    typeVar = self.Cxx.getVar('type')
    iorVar = self.Cxx.getVar('ior')
    baseIorVar = self.Cxx.getVar('baseIor')
    refTypeVar = self.Cxx.getVar('refType')
    returnVar = self.Cxx.getVar('return_value')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.getObjectType())])
    selfVar = self.Cxx.getVar('self')
    allocFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    allocFunc.setChildren([typeVar])
    allocFunc.setMember('tp_alloc')
    allocFunc.setPointer(1)
    allocCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    allocCall.setChildren([allocFunc])
    allocCall.setArguments([typeVar, self.Cxx.getInteger(0)])
    selfInit = ASE.Compiler.Cxx.Initializer.Initializer()
    selfInit.setChildren([self.Cxx.castToType(allocCall, self.getSIDLObjectType(interface))])
    selfDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    selfDecl.setIdentifier(selfVar.getIdentifier())
    selfDecl.setType(self.getSIDLObjectType(interface))
    selfDecl.setInitializer(selfInit)
    selfWeakRefList = ASE.Compiler.Cxx.StructReference.StructReference()
    selfWeakRefList.setChildren([selfVar])
    selfWeakRefList.setMember('weakRefList')
    selfWeakRefList.setPointer(1)
    setWeakRefList = ASE.Compiler.Cxx.Assignment.Assignment()
    setWeakRefList.setChildren([selfWeakRefList, self.Cxx.getNull()])
    selfIor = ASE.Compiler.Cxx.StructReference.StructReference()
    selfIor.setChildren([selfVar])
    selfIor.setMember('ior')
    selfIor.setPointer(1)
    iorObj = ASE.Compiler.Cxx.StructReference.StructReference()
    iorObj.setChildren([iorVar])
    iorObj.setMember('d_object')
    iorObj.setPointer(1)
    selfIorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfIorEpv.setChildren([baseIorVar])
    selfIorEpv.setMember('d_epv')
    selfIorEpv.setPointer(1)
    trueIor = ASE.Compiler.Cxx.StructReference.StructReference()
    trueIor.setChildren([baseIorVar])
    trueIor.setMember('d_object')
    trueIor.setPointer(1)
    iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    iorEpv.setChildren([iorVar])
    iorEpv.setMember('d_epv')
    iorEpv.setPointer(1)
    castFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    castFuncPtr.setChildren([iorEpv])
    castFuncPtr.setMember('castObject')
    castFuncPtr.setPointer(1)
    castFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    castFunc.setChildren([self.Cxx.getGroup(castFuncPtr)])
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFunc)])
    castCall.setArguments([iorObj, self.Cxx.getString(self.SIDL.getBaseParent(interface).getFullIdentifier()), self.Cxx.getNull()])
    baseIorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    baseIorInit.setChildren([castCall])
    typeName = ASE.Compiler.Cxx.StructReference.StructReference()
    typeName.setChildren([typeVar])
    typeName.setMember('tp_name')
    typeName.setPointer(1)
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFunc)])
    castCall.setArguments([iorObj, typeName, self.Cxx.getNull()])
    setSelfIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setSelfIor.setChildren([selfIor, castCall])
    selfRefType = ASE.Compiler.Cxx.StructReference.StructReference()
    selfRefType.setChildren([selfVar])
    selfRefType.setMember('refType')
    selfRefType.setPointer(1)
    setRefType = ASE.Compiler.Cxx.Assignment.Assignment()
    setRefType.setChildren([selfRefType, refTypeVar])
    isNewRef = ASE.Compiler.Cxx.Equality.Equality()
    isNewRef.setChildren([refTypeVar, self.Cxx.getVar('NEW_REF')])
    increfFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    increfFuncPtr.setChildren([selfIorEpv])
    increfFuncPtr.setMember('addReference')
    increfFuncPtr.setPointer(1)
    increfFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    increfFunc.setChildren([self.Cxx.getGroup(increfFuncPtr)])
    increfCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfCall.setChildren([self.Cxx.getGroup(increfFunc)])
    increfCall.setArguments([trueIor, self.Cxx.getNull()])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(increfCall)])
    checkNewRef = ASE.Compiler.Cxx.If.If()
    checkNewRef.setBranch(isNewRef)
    checkNewRef.setChildren([compound])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.addDeclarations([self.Cxx.getDeclaration(baseIorVar, self.IOR.getType(self.SIDL.getBaseParent(interface)), baseIorInit)])
    trueBranch.addChildren([self.Cxx.getExpStmt(setSelfIor), self.Cxx.getExpStmt(setRefType)])
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      selfIorImpl = ASE.Compiler.Cxx.StructReference.StructReference()
      selfIorImpl.setChildren([baseIorVar])
      selfIorImpl.setMember('d_data')
      selfIorImpl.setPointer(1)
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([checkNewRef])
      checkRef = ASE.Compiler.Cxx.If.If()
      checkRef.setBranch(selfIorImpl)
      checkRef.setChildren([compound])
      checkRef.setComments([self.Cxx.getComment('Allow objects without an attached implementation (used in the skeleton constructor)')])
      trueBranch.addChildren([checkRef])
    else:
      trueBranch.addChildren([checkNewRef])
    setRetSelf = ASE.Compiler.Cxx.Assignment.Assignment()
    setRetSelf.setChildren([returnVar, self.Cxx.castToType(selfVar, self.getObjectType())])
    trueBranch.addChildren([self.Cxx.getExpStmt(setRetSelf)])
    setError = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setError.setChildren([self.Cxx.getVar('PyErr_SetString')])
    setError.setArguments([self.Cxx.getVar('PyExc_MemoryError'), self.Cxx.castToString('Unable to allocate interface object')])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(setError)])
    checkSelf = ASE.Compiler.Cxx.If.If()
    checkSelf.setBranch(selfVar)
    checkSelf.setChildren([trueBranch, falseBranch])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setDeclarations([self.Cxx.getDecl(selfDecl)])
    trueBranch.setChildren([self.Cxx.getExpStmt(setWeakRefList), checkSelf])
    setRetNone = ASE.Compiler.Cxx.Assignment.Assignment()
    setRetNone.setChildren([returnVar, self.Cxx.getVar('Py_None')])
    increfReturn = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfReturn.setChildren([self.Cxx.getVar('Py_INCREF')])
    increfReturn.setArguments([returnVar])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(setRetNone), self.Cxx.getExpStmt(increfReturn)])
    checkIor = ASE.Compiler.Cxx.If.If()
    checkIor.setBranch(iorVar)
    checkIor.setChildren([trueBranch, falseBranch])
    returnObj = ASE.Compiler.Cxx.Return.Return()
    returnObj.setChildren([returnVar])
    body.addChildren([checkIor, returnObj])
    typeParam = ASE.Compiler.Cxx.Parameter.Parameter()
    typeParam.setIdentifier(typeVar.getIdentifier())
    typeParam.setType(self.getTypeType())
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(self.IOR.getType(self.SIDL.getBaseInterface(interface)))
    refTypeParam = ASE.Compiler.Cxx.Parameter.Parameter()
    refTypeParam.setIdentifier(refTypeVar.getIdentifier())
    refTypeParam.setType(self.getRefTypeType())
    newName = ASE.Compiler.Cxx.Declarator.Declarator()
    newName.setIdentifier('sidl_new')
    newFunc = ASE.Compiler.Cxx.Function.Function()
    newFunc.setChildren([newName])
    newFunc.setType(self.getObjectType())
    newFunc.setParameters([typeParam, iorParam, refTypeParam])
    newFunc.setBody(body)
    newFunc.setStatic(1)
    return newFunc
  
  def getExtractOpaque(self, type):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Equality
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    if type in self.extractIOR:
      return self.extractIOR[type]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar('obj')
    iorVar = self.Cxx.getVar('ptr')
    checkCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    checkCall.setChildren([self.Cxx.getVar('PyCObject_Check')])
    checkCall.setArguments([objVar])
    ptrInd = ASE.Compiler.Cxx.Indirection.Indirection()
    ptrInd.setChildren([iorVar])
    convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    convertCall.setChildren([self.Cxx.getVar('PyCObject_AsVoidPtr')])
    convertCall.setArguments([objVar])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([ptrInd, convertCall])
    returnTrue = ASE.Compiler.Cxx.Return.Return()
    returnTrue.setChildren([self.Cxx.getInteger(1)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkCObj = ASE.Compiler.Cxx.If.If()
    checkCObj.setBranch(checkCall)
    checkCObj.setChildren([compound])
    hasHandle = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    hasHandle.setChildren([self.Cxx.getVar('PyObject_HasAttrString')])
    hasHandle.setArguments([objVar, self.Cxx.getString('handle')])
    handle = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    handle.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    handle.setArguments([objVar, self.Cxx.getString('handle')])
    convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    convertCall.setChildren([self.Cxx.getVar('PyLong_AsVoidPtr')])
    convertCall.setArguments([handle])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([ptrInd, convertCall])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkHandle = ASE.Compiler.Cxx.If.If()
    checkHandle.setBranch(hasHandle)
    checkHandle.setChildren([compound])
    checkHandle.setComments([self.Cxx.getComment('Why in the hell is there no Python interface to CObject?')])
    isNone = ASE.Compiler.Cxx.Equality.Equality()
    isNone.setChildren([objVar, self.Cxx.getVar('Py_None')])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([ptrInd, self.Cxx.getInteger(0)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkNone = ASE.Compiler.Cxx.If.If()
    checkNone.setBranch(isNone)
    checkNone.setChildren([compound])
    returnFalse = ASE.Compiler.Cxx.Return.Return()
    returnFalse.setChildren([self.Cxx.getInteger(0)])
    body.addChildren([checkCObj, checkHandle, checkNone, returnFalse])
    objParam = ASE.Compiler.Cxx.Parameter.Parameter()
    objParam.setIdentifier(objVar.getIdentifier())
    objParam.setType(self.getObjectType())
    iorPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorPtrType.setChildren([self.IOR.getType(type)])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(iorPtrType)
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__extract_ior')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([objParam, iorParam])
    extractFunc.setBody(body)
    if not type in self.extractIOR:
      self.extractIOR[type] = extractFunc
    return extractFunc
  
  def getWrapOpaque(self, type):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    if type in self.wrapIOR:
      return self.wrapIOR[type]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    iorVar = self.Cxx.getVar('ptr')
    noneVar = self.Cxx.getVar('Py_None')
    convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    convertCall.setChildren([self.Cxx.getVar('PyCObject_FromVoidPtr')])
    convertCall.setArguments([iorVar, self.Cxx.getNull()])
    returnPtr = ASE.Compiler.Cxx.Return.Return()
    returnPtr.setChildren([convertCall])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([returnPtr])
    increfNone = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfNone.setChildren([self.Cxx.getVar('Py_INCREF')])
    increfNone.setArguments([noneVar])
    returnNone = ASE.Compiler.Cxx.Return.Return()
    returnNone.setChildren([noneVar])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(increfNone), returnNone])
    checkPtr = ASE.Compiler.Cxx.If.If()
    checkPtr.setBranch(iorVar)
    checkPtr.setChildren([trueBranch, falseBranch])
    body.addChildren([checkPtr])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(self.IOR.getType(type))
    wrapName = ASE.Compiler.Cxx.Declarator.Declarator()
    wrapName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__wrap_ior')
    wrapFunc = ASE.Compiler.Cxx.Function.Function()
    wrapFunc.setChildren([wrapName])
    wrapFunc.setType(self.getObjectType())
    wrapFunc.setStatic(1)
    wrapFunc.setParameters([iorParam])
    wrapFunc.setBody(body)
    self.wrapIOR[type] = wrapFunc
    return wrapFunc
  
  def getExtractIOR(self, interface, moduleInterface = None):
    '''
    Generate the function which extracts the IOR pointer from a Python object of the given interface
    '''
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Constant
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Equality
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    if interface in self.extractIOR:
      if moduleInterface is None:
        if isinstance(self.extractIOR[interface], dict):
          return self.extractIOR[interface].values()[0]
        else:
          return self.extractIOR[interface]
      elif moduleInterface in self.extractIOR[interface]:
        return self.extractIOR[interface][moduleInterface]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar('obj')
    iorVar = self.Cxx.getVar('ior')
    objIorVar = self.Cxx.getVar('objIor')
    moduleVar = self.Cxx.getVar('module')
    typeVar = self.Cxx.getVar('type')
    baseType = self.IOR.getType(self.SIDL.getBaseInterface(interface))
    objIor = ASE.Compiler.Cxx.StructReference.StructReference()
    objIor.setChildren([objVar])
    objIor.setMember('ior')
    objIor.setPointer(1)
    objIorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    objIorInit.setChildren([self.Cxx.castToType(objIor, baseType)])
    objIorDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    objIorDecl.setIdentifier(objIorVar.getIdentifier())
    objIorDecl.setType(baseType)
    objIorDecl.setInitializer(objIorInit)
    body.addDeclarations([self.Cxx.getDecl(objIorDecl)])
    objEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    objEpv.setChildren([objIorVar])
    objEpv.setMember('d_epv')
    objEpv.setPointer(1)
    trueIor = ASE.Compiler.Cxx.StructReference.StructReference()
    trueIor.setChildren([objIorVar])
    trueIor.setMember('d_object')
    trueIor.setPointer(1)
    noneCmp = ASE.Compiler.Cxx.Equality.Equality()
    noneCmp.setChildren([self.Cxx.castToType(objVar, self.getObjectType()), self.Cxx.getVar('Py_None')])
    iorRef = ASE.Compiler.Cxx.Indirection.Indirection()
    iorRef.setChildren([iorVar])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([iorRef, self.Cxx.getNull()])
    returnTrue = ASE.Compiler.Cxx.Return.Return()
    returnTrue.setChildren([self.Cxx.getInteger(1)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setIor), returnTrue])
    checkNone = ASE.Compiler.Cxx.If.If()
    checkNone.setBranch(noneCmp)
    checkNone.setChildren([compound])
    objType = ASE.Compiler.Cxx.StructReference.StructReference()
    objType.setChildren([objVar])
    objType.setMember('ob_type')
    objType.setPointer(1)
    branch = ASE.Compiler.Cxx.Inequality.Inequality()
    branch.setChildren([objType, typeVar])
    castFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    castFuncPtr.setChildren([objEpv])
    castFuncPtr.setMember('castObject')
    castFuncPtr.setPointer(1)
    castFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    castFunc.setChildren([self.Cxx.getGroup(castFuncPtr)])
    interfaceName = ASE.Compiler.Cxx.Constant.Constant()
    interfaceName.setIdentifier(interface.getFullIdentifier())
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFunc)])
    castCall.setArguments([trueIor, interfaceName, self.Cxx.getNull()])
    setIor = ASE.Compiler.Cxx.Assignment.Assignment()
    setIor.setChildren([iorRef, castCall])
    body.addChildren([checkNone, self.Cxx.getExpStmt(setIor), self.getErrorCheck('PyExc_TypeError', 'Argument is not of type ' + interface.getFullIdentifier(), self.Cxx.getGroup(iorRef), self.Cxx.getInteger(0)), returnTrue])
    objParam = ASE.Compiler.Cxx.Parameter.Parameter()
    objParam.setIdentifier(objVar.getIdentifier())
    objParam.setType(self.getSIDLObjectType(moduleInterface))
    iorPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorPtrType.setChildren([self.IOR.getType(interface)])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(iorPtrType)
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__extract_ior')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([objParam, iorParam])
    extractFunc.setBody(body)
    if not interface in self.extractIOR:
      self.extractIOR[interface] = {}

    self.extractIOR[interface][moduleInterface] = extractFunc
    return extractFunc
  
  def getWrapIOR(self, interface, isWeak = 0):
    '''
    Generate the function which wraps an IOR pointer in a Python object of the given interface
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    if not isWeak and interface in self.wrapIOR:
      return self.wrapIOR[interface]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    iorVar = self.Cxx.getVar('ior')
    moduleVar = self.Cxx.getVar('module')
    typeVar = self.Cxx.getVar('type')
    importCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    importCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
    importCall.setArguments([self.Cxx.castToString(interface.getFullIdentifier())])
    moduleInit = ASE.Compiler.Cxx.Initializer.Initializer()
    moduleInit.setChildren([importCall])
    moduleDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    moduleDecl.setIdentifier(moduleVar.getIdentifier())
    moduleDecl.setType(self.getObjectType())
    moduleDecl.setInitializer(moduleInit)
    getAttrCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getAttrCall.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    getAttrCall.setArguments([moduleVar, self.Cxx.castToString(interface.getIdentifier())])
    typeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    typeInit.setChildren([self.Cxx.castToType(getAttrCall, self.getTypeType())])
    typeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    typeDecl.setIdentifier(typeVar.getIdentifier())
    typeDecl.setType(self.getTypeType())
    typeDecl.setInitializer(typeInit)
    body.addDeclarations([self.Cxx.getDecl(moduleDecl), self.Cxx.getDecl(typeDecl)])
    newCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    newCall.setChildren([self.Cxx.getVar('sidl_new')])
    newCall.addArguments([typeVar, self.Cxx.castToType(iorVar, self.IOR.getType(self.SIDL.getBaseInterface(interface)))])
    if isWeak:
      newCall.addArguments([self.Cxx.getVar('WEAK_REF')])
    else:
      newCall.addArguments([self.Cxx.getVar('STOLEN_REF')])
    returnNew = ASE.Compiler.Cxx.Return.Return()
    returnNew.setChildren([newCall])
    body.addChildren([self.getErrorCheck('PyExc_ImportError', 'Could not import module ' + interface.getFullIdentifier(), moduleVar, self.Cxx.getInteger(0)), self.getErrorCheck('PyExc_TypeError', 'Could not find type ' + interface.getFullIdentifier(), typeVar, self.Cxx.getInteger(0)), returnNew])
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(iorVar.getIdentifier())
    iorParam.setType(self.IOR.getType(interface))
    wrapName = ASE.Compiler.Cxx.Declarator.Declarator()
    if isWeak:
      wrapName.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__wrap_ior_weak')
    else:
      wrapName.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__wrap_ior')
    wrapFunc = ASE.Compiler.Cxx.Function.Function()
    wrapFunc.setChildren([wrapName])
    wrapFunc.setType(self.getObjectType())
    wrapFunc.setStatic(1)
    wrapFunc.setParameters([iorParam])
    wrapFunc.setBody(body)
    if not isWeak:
      self.wrapIOR[interface] = wrapFunc
    return wrapFunc
  
  def getPythonArrayInfo(self, body):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Subtraction
    pyArrayVar = self.Cxx.getVar('pyArray')
    dimenVar = self.Cxx.getVar('dimen')
    indLowerVar = self.Cxx.getVar('indLower')
    indUpperVar = self.Cxx.getVar('indUpper')
    indStrideVar = self.Cxx.getVar('indStride')
    lowerVar = self.Cxx.getVar('lower')
    upperVar = self.Cxx.getVar('upper')
    strideVar = self.Cxx.getVar('stride')
    dVar = self.Cxx.getVar('d')
    freeVar = self.Cxx.getVar('free')
    intDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.IOR.typeMap['int'])
    sizeofInt = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    intArray = ASE.Compiler.SIDL.Array.Array()
    intArray.setType(self.SIDL.typeMap['int'])
    intStruct = ASE.Compiler.Cxx.Struct.Struct(self.IOR.getType(intArray).getChildren()[0])
    pyArrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayDimen.setChildren([pyArrayVar])
    pyArrayDimen.setMember('nd')
    pyArrayDimen.setPointer(1)
    pyArrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayUpper.setChildren([pyArrayVar])
    pyArrayUpper.setMember('dimensions')
    pyArrayUpper.setPointer(1)
    pyArrayStride = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayStride.setChildren([pyArrayVar])
    pyArrayStride.setMember('strides')
    pyArrayStride.setPointer(1)
    pyArrayDesc = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayDesc.setChildren([pyArrayVar])
    pyArrayDesc.setMember('descr')
    pyArrayDesc.setPointer(1)
    pyArrayElsize = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayElsize.setChildren([pyArrayDesc])
    pyArrayElsize.setMember('elsize')
    pyArrayElsize.setPointer(1)
    dPyArrayUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dPyArrayUpper.setChildren([pyArrayUpper])
    dPyArrayUpper.setIndex(dVar)
    dPyArrayStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dPyArrayStride.setChildren([pyArrayStride])
    dPyArrayStride.setIndex(dVar)
    lowerFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    upperFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    strideFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    strideFirst.setChildren([strideVar])
    strideFirst.setMember('d_firstElement')
    dLowerFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dLowerFirst.setChildren([lowerFirst])
    dLowerFirst.setIndex(dVar)
    dUpperFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dUpperFirst.setChildren([upperFirst])
    dUpperFirst.setIndex(dVar)
    dStrideFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dStrideFirst.setChildren([strideFirst])
    dStrideFirst.setIndex(dVar)
    dimenInit = ASE.Compiler.Cxx.Initializer.Initializer()
    dimenInit.setChildren([pyArrayDimen])
    indLowerInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indLowerInit.setChildren([self.Cxx.getInteger(0)])
    indLowerInit.setList(1)
    indLower = ASE.Compiler.Cxx.Array.Array()
    indLower.setChildren([indLowerVar])
    indLower.setType(self.IOR.typeMap['int'])
    indLower.setSize(self.Cxx.getInteger(1))
    indLower.setInitializer(indLowerInit)
    dimenMinus1 = ASE.Compiler.Cxx.Subtraction.Subtraction()
    dimenMinus1.setChildren([dimenVar, self.Cxx.getInteger(1)])
    indUpperInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indUpperInit.setChildren([dimenMinus1])
    indUpperInit.setList(1)
    indUpper = ASE.Compiler.Cxx.Array.Array()
    indUpper.setChildren([indUpperVar])
    indUpper.setType(self.IOR.typeMap['int'])
    indUpper.setSize(self.Cxx.getInteger(1))
    indUpper.setInitializer(indUpperInit)
    indStrideInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indStrideInit.setChildren([self.Cxx.getInteger(1)])
    indStrideInit.setList(1)
    indStride = ASE.Compiler.Cxx.Array.Array()
    indStride.setChildren([indStrideVar])
    indStride.setType(self.IOR.typeMap['int'])
    indStride.setSize(self.Cxx.getInteger(1))
    indStride.setInitializer(indStrideInit)
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    indexSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indexSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    indexAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    indexAlloc.setChildren([self.Cxx.getVar('malloc')])
    indexAlloc.setArguments([indexSize])
    indexInit = ASE.Compiler.Cxx.Initializer.Initializer()
    indexInit.setChildren([indexAlloc, indLowerVar, indUpperVar, indStrideVar, self.Cxx.getInteger(0), self.Cxx.getInteger(1)])
    body.addDeclarations([self.Cxx.getDeclaration(dimenVar, self.IOR.typeMap['int'], dimenInit)])
    body.addDeclarations(map(self.Cxx.getDecl, [indLower, indUpper, indStride]))
    body.addDeclarations([self.Cxx.getDeclaration(lowerVar, intStruct, indexInit), self.Cxx.getDeclaration(upperVar, intStruct, indexInit), self.Cxx.getDeclaration(strideVar, intStruct, indexInit), self.Cxx.getDeclaration(dVar, self.IOR.typeMap['int'])])
    setLower = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([dLowerFirst, self.Cxx.getInteger(0)])
    newUpper = ASE.Compiler.Cxx.Subtraction.Subtraction()
    newUpper.setChildren([dPyArrayUpper, self.Cxx.getInteger(1)])
    setUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([dUpperFirst, newUpper])
    newStride = ASE.Compiler.Cxx.Division.Division()
    newStride.setChildren([dPyArrayStride, pyArrayElsize])
    setStride = ASE.Compiler.Cxx.Assignment.Assignment()
    setStride.setChildren([dStrideFirst, newStride])
    freeLower = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeLower.setChildren([freeVar])
    freeLower.setArguments([lowerFirst])
    freeUpper = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeUpper.setChildren([freeVar])
    freeUpper.setArguments([upperFirst])
    freeStride = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeStride.setChildren([freeVar])
    freeStride.setArguments([strideFirst])
    returnFail = ASE.Compiler.Cxx.Return.Return()
    returnFail.setChildren([self.Cxx.getInteger(0)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addChildren(map(self.Cxx.getExpStmt, [freeLower, freeUpper, freeStride]))
    compound.addChildren([returnFail])
    extraSpace = ASE.Compiler.Cxx.Modulo.Modulo()
    extraSpace.setChildren([dPyArrayStride, pyArrayElsize])
    checkSize = ASE.Compiler.Cxx.If.If()
    checkSize.setBranch(extraSpace)
    checkSize.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren(map(self.Cxx.getExpStmt, [setLower, setUpper, setStride]) + [checkSize])
    dInit = ASE.Compiler.Cxx.Assignment.Assignment()
    dInit.setChildren([dVar, self.Cxx.getInteger(0)])
    dCmp = ASE.Compiler.Cxx.LessThan.LessThan()
    dCmp.setChildren([dVar, dimenVar])
    dInc = ASE.Compiler.Cxx.Increment.Increment()
    dInc.setChildren([dVar])
    dInc.setPostfix(1)
    indexLoop = ASE.Compiler.Cxx.For.For()
    indexLoop.setInitialization(dInit)
    indexLoop.setBranch(dCmp)
    indexLoop.setIncrement(dInc)
    indexLoop.setChildren([compound])
    body.addChildren([indexLoop])
    return

  def getExtractArray(self, type, moduleInterface = None, borrow = 0):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.AdditionAssignment
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Break
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.SubtractionAssignment
    if borrow:
      if not type.getBaseType() or type.getIdentifier() in ['opaque', 'string']:
        borrow = 0
    if type in self.extractArray:
      if moduleInterface in self.extractArray[type][borrow]:
        return self.extractArray[type][borrow][moduleInterface]
      elif moduleInterface is None and len(self.extractArray[type][borrow].keys()):
        return self.extractArray[type][borrow].values()[0]
    else:
      self.extractArray[type] = [{}, {}]
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar('obj')
    arrayVar = self.Cxx.getVar('array')
    pyArrayVar = self.Cxx.getVar('pyArray')
    returnVar = self.Cxx.getVar('_return_value')
    arrayType = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(type)
    intArray = ASE.Compiler.SIDL.Array.Array()
    intArray.setType(self.SIDL.typeMap['int'])
    getPyArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getPyArray.setChildren([self.Cxx.getVar('PyArray_FromObject')])
    getPyArray.setArguments([objVar, self.getNumericTypeCode(type), self.Cxx.getInteger(0), self.Cxx.getInteger(0)])
    pyArrayInit = ASE.Compiler.Cxx.Initializer.Initializer()
    pyArrayInit.setChildren([self.Cxx.castToType(getPyArray, self.getArrayType())])
    pyArrayDecl = self.Cxx.getDeclaration(pyArrayVar, self.getArrayType(), pyArrayInit, 'This will create a new Numeric array from a Python sequence')
    returnDecl = self.Cxx.getDeclaration(returnVar, self.typeMap['int'])
    body.addDeclarations([pyArrayDecl, returnDecl])
    dimenVar = self.Cxx.getVar('dimen')
    lowerVar = self.Cxx.getVar('lower')
    upperVar = self.Cxx.getVar('upper')
    strideVar = self.Cxx.getVar('stride')
    freeVar = self.Cxx.getVar('free')
    intDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.typeMap['int'])
    intPtr = ASE.Compiler.Cxx.Pointer.Pointer()
    intPtr.setChildren([self.typeMap['int']])
    charPtr = ASE.Compiler.Cxx.Pointer.Pointer()
    charPtr.setChildren([self.typeMap['char']])
    arrayInd = ASE.Compiler.Cxx.Indirection.Indirection()
    arrayInd.setChildren([arrayVar])
    arrayInit = ASE.Compiler.Cxx.Assignment.Assignment()
    arrayInit.setChildren([arrayInd, self.Cxx.getNull()])
    arrayInit = self.Cxx.getExpStmt(arrayInit)
    lowerFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    upperFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    strideFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    strideFirst.setChildren([strideVar])
    strideFirst.setMember('d_firstElement')
    mainBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    self.getPythonArrayInfo(mainBody)
    externalsVar = self.Cxx.getVar('_externals')
    extFuncVar = self.Cxx.getVar('_extFunc')
    sizeVar = self.Cxx.getVar('size')
    indicesVar = self.Cxx.getVar('indices')
    dataVar = self.Cxx.getVar('data')
    iVar = self.Cxx.getVar('i')
    dVar = self.Cxx.getVar('d')
    freejVar = self.Cxx.getVar('free')

    if type.getBaseType():
      externalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      externalsCall.setChildren([self.Cxx.getVar(type.getFullIdentifier().replace('.', '_')+ '__externals')])
    else:
      externalsFuncType = ASE.Compiler.Cxx.Function.Function()
      externalsFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      externalsFuncType.setType(self.IOR.getExternalType(type))
      sidlExtCall       = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
      staticEPV         = ASE.Compiler.Cxx.StructReference.StructReference()
      staticEPV.setChildren([sidlExtCall])
      staticEPV.setMember('getStaticEPV')
      staticEPV.setPointer(1)
      staticEPVCall     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      staticEPVCall.setChildren([staticEPV])
      staticEPVCall.setArguments([self.Cxx.getNull()])
      classMethods      = ASE.Compiler.Cxx.StructReference.StructReference()
      classMethods.setChildren([staticEPVCall])
      classMethods.setMember('getClassMethodTable')
      classMethods.setPointer(1)
      classMethodsCall  = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      classMethodsCall.setChildren([classMethods])
      classMethodsCall.setArguments([self.Cxx.getString(type.getFullIdentifier()), self.Cxx.getNull()])
      externalsFunc     = ASE.Compiler.Cxx.Cast.Cast()
      externalsFunc.setTypeName(externalsFuncType)
      externalsFunc.setChildren([classMethodsCall])
      externalsInit = ASE.Compiler.Cxx.Initializer.Initializer()
      externalsInit.setChildren([externalsFunc])
      externalsFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      externalsFuncPtr.setIdentifier(externalsVar.getIdentifier())
      externalsFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      externalsFunc = ASE.Compiler.Cxx.Function.Function()
      externalsFunc.setChildren([self.Cxx.getDeclGroup(externalsFuncPtr)])
      externalsFunc.setType(self.IOR.getExternalType(type))
      externalsFunc.setInitializer(externalsInit)
      mainBody.addDeclarations([self.Cxx.getDecl(externalsFunc)])
      externalsInd  = ASE.Compiler.Cxx.Indirection.Indirection()
      externalsInd.setChildren([externalsVar])
      externalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      externalsCall.setChildren([self.Cxx.getGroup(externalsInd)])
    extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
    extFuncInit.setChildren([externalsCall])
    extFuncDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    extFuncDecl.setIdentifier(extFuncVar.getIdentifier())
    extFuncDecl.setType(self.IOR.getExternalType(type))
    extFuncDecl.setInitializer(extFuncInit)
    arraySize = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    arraySize.setChildren([self.Cxx.getVar('PyArray_Size')])
    arraySize.setArguments([self.Cxx.castToType(pyArrayVar, self.getObjectType())])
    sizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    sizeInit.setChildren([arraySize])
    sizeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    sizeDecl.setIdentifier(sizeVar.getIdentifier())
    sizeDecl.setType(self.typeMap['int'])
    sizeDecl.setInitializer(sizeInit)
    sizeofInt = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    indicesSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indicesSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    indicesMalloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    indicesMalloc.setChildren([self.Cxx.getVar('malloc')])
    indicesMalloc.setArguments([indicesSize])
    pyArrayData = ASE.Compiler.Cxx.StructReference.StructReference()
    pyArrayData.setChildren([pyArrayVar])
    pyArrayData.setMember('data')
    pyArrayData.setPointer(1)
    dataInit = ASE.Compiler.Cxx.Initializer.Initializer()
    dataInit.setChildren([pyArrayData])
    dataDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    dataDecl.setIdentifier(dataVar.getIdentifier())
    dataDecl.setType(charPtr)
    dataDecl.setInitializer(dataInit)
    iDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    iDecl.setIdentifier(iVar.getIdentifier())
    iDecl.setType(self.typeMap['int'])
    if not borrow:
      createFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      createFuncPtr.setChildren([extFuncVar])
      createFuncPtr.setMember('createArray')
      createFuncPtr.setPointer(1)
      createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      createFunc.setChildren([createFuncPtr])
      createArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createArray.setChildren([self.Cxx.getGroup(createFunc)])
      createArray.setArguments([dimenVar, self.Cxx.getAddress(lowerVar), self.Cxx.getAddress(upperVar)])
      createArray.addArguments([self.Cxx.getNull()])
      setArray = ASE.Compiler.Cxx.Assignment.Assignment()
      setArray.setChildren([arrayInd, createArray])
    else:
      borrowFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      borrowFuncPtr.setChildren([extFuncVar])
      borrowFuncPtr.setMember('borrowArray')
      borrowFuncPtr.setPointer(1)
      borrowFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      borrowFunc.setChildren([borrowFuncPtr])
      borrowArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      borrowArray.setChildren([self.Cxx.getGroup(borrowFunc)])
      borrowArray.setArguments([dimenVar, self.Cxx.getAddress(lowerVar), self.Cxx.getAddress(upperVar), self.Cxx.getAddress(strideVar), dataVar])
      borrowArray.addArguments([self.Cxx.getNull()])
      setArray = ASE.Compiler.Cxx.Assignment.Assignment()
      setArray.setChildren([arrayInd, borrowArray])
    freeLower = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeLower.setChildren([freeVar])
    freeLower.setArguments([lowerFirst])
    freeUpper = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeUpper.setChildren([freeVar])
    freeUpper.setArguments([upperFirst])
    freeStride = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeStride.setChildren([freeVar])
    freeStride.setArguments([strideFirst])
    indData = ASE.Compiler.Cxx.StructReference.StructReference()
    indData.setChildren([indicesVar])
    indData.setMember('d_firstElement')
    mainBody.addDeclarations(map(self.Cxx.getDecl, [extFuncDecl, sizeDecl, dataDecl, iDecl]))
    mainBody.addChildren(map(self.Cxx.getExpStmt, [setArray, freeLower, freeUpper, freeStride]))
    if not borrow:
      indLowerVar = self.Cxx.getVar('indLower')
      indUpperVar = self.Cxx.getVar('indUpper')
      indStrideVar = self.Cxx.getVar('indStride')
      indicesInit = ASE.Compiler.Cxx.Initializer.Initializer()
      indicesInit.setChildren([indicesMalloc, indLowerVar, indUpperVar, indStrideVar, self.Cxx.getInteger(0), self.Cxx.getInteger(1)])
      indicesDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      indicesDecl.setIdentifier(indicesVar.getIdentifier())
      indicesDecl.setType(self.IOR.getType(intArray).getChildren()[0])
      indicesDecl.setInitializer(indicesInit)
      mainBody.addDeclarations(map(self.Cxx.getDecl, [indicesDecl]))
      clearIndices = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      clearIndices.setChildren([self.Cxx.getVar('memset')])
      clearIndices.setArguments([indData, self.Cxx.getInteger(0), indicesSize])
      mainBody.addChildren(map(self.Cxx.getExpStmt, [clearIndices]))
      elemVar = self.Cxx.getVar('elem')
      elemDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      elemDecl.setIdentifier(elemVar.getIdentifier())
      elemDecl.setType(self.IOR.getType(type))
      destroyFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      destroyFuncPtr.setChildren([extFuncVar])
      destroyFuncPtr.setMember('destroyArray')
      destroyFuncPtr.setPointer(1)
      destroyFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      destroyFunc.setChildren([destroyFuncPtr])
      destroyArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      destroyArray.setChildren([self.Cxx.getGroup(destroyFunc)])
      destroyArray.setArguments([arrayInd, self.Cxx.getNull()])
      gotoError = ASE.Compiler.Cxx.Goto.Goto()
      gotoError.setIdentifier('error')
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setDeclarations([self.Cxx.getDecl(elemDecl)])
      if type.getBaseType():
        typeName = type.getIdentifier()
        if typeName == 'string':
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.getObjectType(), 1)])
          cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
          cmpd.setChildren([self.Cxx.getExpStmt(destroyArray), arrayInit, gotoError])
          convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          convertCall.setChildren([self.Cxx.getVar('PyString_AsString')])
          convertCall.setArguments([dataInd])
          setElem = ASE.Compiler.Cxx.Assignment.Assignment()
          setElem.setChildren([elemVar, convertCall])
          negConvert = ASE.Compiler.Cxx.Negation.Negation()
          negConvert.setChildren([self.Cxx.getGroup(setElem)])
          convertElem = ASE.Compiler.Cxx.If.If()
          convertElem.setBranch(negConvert)
          convertElem.setChildren([cmpd])
          compound.addChildren([convertElem])
          self.copyValue_string(elemVar, compound)
        elif typeName == 'opaque':
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.getObjectType(), 1)])
          cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
          cmpd.setChildren([self.Cxx.getExpStmt(destroyArray), arrayInit, gotoError])
          convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          convertCall.setChildren([self.Cxx.getVar('PyCObject_AsVoidPtr')])
          convertCall.setArguments([dataInd])
          setElem = ASE.Compiler.Cxx.Assignment.Assignment()
          setElem.setChildren([elemVar, convertCall])
          negConvert = ASE.Compiler.Cxx.Negation.Negation()
          negConvert.setChildren([self.Cxx.getGroup(setElem)])
          convertElem = ASE.Compiler.Cxx.If.If()
          convertElem.setBranch(negConvert)
          convertElem.setChildren([cmpd])
          compound.addChildren([convertElem])
        elif typeName in ['dcomplex', 'fcomplex']:
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.IOR.getType(type), 1)])
          self.setValue_dcomplex(elemVar, dataInd, compound, setIOR = 1, pythonImagName = 'imaginary')
        else:
          dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
          dataInd.setChildren([self.Cxx.castToType(dataVar, self.getType(type), 1)])
          convertElem = ASE.Compiler.Cxx.Assignment.Assignment()
          convertElem.setChildren([elemVar, dataInd])
          compound.addChildren([self.Cxx.getExpStmt(convertElem)])
      elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        dataInd = ASE.Compiler.Cxx.Indirection.Indirection()
        dataInd.setChildren([self.Cxx.castToType(dataVar, self.getObjectType(), 1)])
        cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        cmpd.setChildren([self.Cxx.getExpStmt(destroyArray), arrayInit, gotoError])
        convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        convertCall.setChildren([self.Cxx.getVar(type.getFullIdentifier().replace('.', '_') + '__extract_ior')])
        convertCall.setArguments([self.Cxx.castToType(dataInd, self.getSIDLObjectType(moduleInterface)), self.Cxx.getAddress(elemVar)])
        negConvert = ASE.Compiler.Cxx.Negation.Negation()
        negConvert.setChildren([convertCall])
        convertElem = ASE.Compiler.Cxx.If.If()
        convertElem.setBranch(negConvert)
        convertElem.setChildren([cmpd])
        compound.addChildren([convertElem])
      setFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      setFuncPtr.setChildren([extFuncVar])
      setFuncPtr.setMember('setElement')
      setFuncPtr.setPointer(1)
      setFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      setFunc.setChildren([setFuncPtr])
      setElement = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      setElement.setChildren([self.Cxx.getGroup(setFunc)])
      setElement.setArguments([arrayInd, self.Cxx.getAddress(indicesVar), elemVar])
      setElement.addArguments([self.Cxx.getNull()])
      strides = ASE.Compiler.Cxx.StructReference.StructReference()
      strides.setChildren([pyArrayVar])
      strides.setMember('strides')
      strides.setPointer(1)
      dimStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dimStride.setChildren([strides])
      dimStride.setIndex(dVar)
      incData = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
      incData.setChildren([dataVar, dimStride])
      lengths = ASE.Compiler.Cxx.StructReference.StructReference()
      lengths.setChildren([pyArrayVar])
      lengths.setMember('dimensions')
      lengths.setPointer(1)
      dimLength = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dimLength.setChildren([lengths])
      dimLength.setIndex(dVar)
      dimIndex = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dimIndex.setChildren([indData])
      dimIndex.setIndex(dVar)
      incDimIndex = ASE.Compiler.Cxx.Increment.Increment()
      incDimIndex.setChildren([self.Cxx.getGroup(dimIndex)])
      compLength = ASE.Compiler.Cxx.LessThan.LessThan()
      compLength.setChildren([incDimIndex, dimLength])
      trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      trueBranch.setChildren([ASE.Compiler.Cxx.Break.Break()])
      resetIndex = ASE.Compiler.Cxx.Assignment.Assignment()
      resetIndex.setChildren([dimIndex, self.Cxx.getInteger(0)])
      dimSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
      dimSize.setChildren([dimLength, dimStride])
      decData = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
      decData.setChildren([dataVar, dimSize])
      falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      falseBranch.setChildren([self.Cxx.getExpStmt(resetIndex), self.Cxx.getExpStmt(decData)])
      checkWrap = ASE.Compiler.Cxx.If.If()
      checkWrap.setBranch(compLength)
      checkWrap.setChildren([trueBranch, falseBranch])
      cmpd = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      cmpd.setChildren([self.Cxx.getExpStmt(incData), checkWrap])
      initDim = ASE.Compiler.Cxx.Assignment.Assignment()
      initDim.setChildren([dVar, self.Cxx.getInteger(0)])
      checkDim = ASE.Compiler.Cxx.LessThan.LessThan()
      checkDim.setChildren([dVar, dimenVar])
      incDim = ASE.Compiler.Cxx.Increment.Increment()
      incDim.setChildren([dVar])
      incDim.setPostfix(1)
      updateLocation = ASE.Compiler.Cxx.For.For()
      updateLocation.setInitialization(initDim)
      updateLocation.setBranch(checkDim)
      updateLocation.setIncrement(incDim)
      updateLocation.setChildren([cmpd])
      compound.addChildren([self.Cxx.getExpStmt(setElement), updateLocation])
      initIndex = ASE.Compiler.Cxx.Assignment.Assignment()
      initIndex.setChildren([iVar, self.Cxx.getInteger(0)])
      checkIndex = ASE.Compiler.Cxx.LessThan.LessThan()
      checkIndex.setChildren([iVar, sizeVar])
      incIndex = ASE.Compiler.Cxx.Increment.Increment()
      incIndex.setChildren([iVar])
      incIndex.setPostfix(1)
      elementLoop = ASE.Compiler.Cxx.For.For()
      elementLoop.setInitialization(initIndex)
      elementLoop.setBranch(checkIndex)
      elementLoop.setIncrement(incIndex)
      elementLoop.setChildren([compound])
      mainBody.addChildren([elementLoop])
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([returnVar, self.Cxx.getInteger(1)])
    mainBody.addChildren([self.Cxx.getExpStmt(setReturn)])
    if not borrow:
      freeIndices = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      freeIndices.setChildren([freeVar])
      freeIndices.setArguments([indData])
      freeIndices = self.Cxx.getExpStmt(freeIndices)
      freeIndices.setIdentifier('error')
      mainBody.addChildren([freeIndices])
      decrefPyArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefPyArray.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefPyArray.setArguments([pyArrayVar])
      mainBody.addChildren([self.Cxx.getExpStmt(decrefPyArray)])
    checkPyArray = ASE.Compiler.Cxx.If.If()
    checkPyArray.setBranch(pyArrayVar)
    checkPyArray.setChildren([mainBody])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    returnStmt.setChildren([returnVar])
    body.addChildren([arrayInit, checkPyArray, returnStmt])
    objParam = ASE.Compiler.Cxx.Parameter.Parameter()
    objParam.setIdentifier(objVar.getIdentifier())
    objParam.setType(self.getObjectType())
    arrayPtrType = ASE.Compiler.Cxx.Pointer.Pointer()
    arrayPtrType.setChildren([self.IOR.getType(arrayType)])
    arrayParam = ASE.Compiler.Cxx.Parameter.Parameter()
    arrayParam.setIdentifier(arrayVar.getIdentifier())
    arrayParam.setType(arrayPtrType)
    extractName = ASE.Compiler.Cxx.Declarator.Declarator()
    extractName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__extract_array')
    extractFunc = ASE.Compiler.Cxx.Function.Function()
    extractFunc.setChildren([extractName])
    extractFunc.setType(self.typeMap['int'])
    extractFunc.setStatic(1)
    extractFunc.setParameters([objParam, arrayParam])
    extractFunc.setBody(body)
    self.extractArray[type][borrow][moduleInterface] = extractFunc
    return extractFunc

  def getWrapArray(self, type):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Return

    if type in self.wrapArray:
      return self.wrapArray[type]
    body          = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    arrayVar      = self.Cxx.getVar('array')
    returnVar     = self.Cxx.getVar('_return_value')

    arrayType     = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(type)
    intArray      = ASE.Compiler.SIDL.Array.Array()
    intArray.setType(self.SIDL.typeMap['int'])
    charPtr       = ASE.Compiler.Cxx.Pointer.Pointer()
    charPtr.setChildren([self.typeMap['char']])

    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.getArrayType())])
    mainBody      = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()

    externalsVar  = self.Cxx.getVar('_externals')
    extFuncVar    = self.Cxx.getVar('_extFunc')
    dimenVar      = self.Cxx.getVar('dimen')
    lowerVar      = self.Cxx.getVar('lower')
    upperVar      = self.Cxx.getVar('upper')
    numElemVar    = self.Cxx.getVar('numElem')
    iVar          = self.Cxx.getVar('i')

    if type.getBaseType():
      externalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      externalsCall.setChildren([self.Cxx.getVar(type.getFullIdentifier().replace('.', '_')+ '__externals')])
    else:
      externalsFuncType = ASE.Compiler.Cxx.Function.Function()
      externalsFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      externalsFuncType.setType(self.IOR.getExternalType(type))
      sidlExtCall       = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
      staticEPV         = ASE.Compiler.Cxx.StructReference.StructReference()
      staticEPV.setChildren([sidlExtCall])
      staticEPV.setMember('getStaticEPV')
      staticEPV.setPointer(1)
      staticEPVCall     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      staticEPVCall.setChildren([staticEPV])
      staticEPVCall.setArguments([self.Cxx.getNull()])
      classMethods      = ASE.Compiler.Cxx.StructReference.StructReference()
      classMethods.setChildren([staticEPVCall])
      classMethods.setMember('getClassMethodTable')
      classMethods.setPointer(1)
      classMethodsCall  = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      classMethodsCall.setChildren([classMethods])
      classMethodsCall.setArguments([self.Cxx.getString(type.getFullIdentifier()), self.Cxx.getNull()])
      externalsFunc     = ASE.Compiler.Cxx.Cast.Cast()
      externalsFunc.setTypeName(externalsFuncType)
      externalsFunc.setChildren([classMethodsCall])
      externalsInit = ASE.Compiler.Cxx.Initializer.Initializer()
      externalsInit.setChildren([externalsFunc])
      externalsFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      externalsFuncPtr.setIdentifier(externalsVar.getIdentifier())
      externalsFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      externalsFunc = ASE.Compiler.Cxx.Function.Function()
      externalsFunc.setChildren([self.Cxx.getDeclGroup(externalsFuncPtr)])
      externalsFunc.setType(self.IOR.getExternalType(type))
      externalsFunc.setInitializer(externalsInit)
      mainBody.addDeclarations([self.Cxx.getDecl(externalsFunc)])
      externalsInd  = ASE.Compiler.Cxx.Indirection.Indirection()
      externalsInd.setChildren([externalsVar])
      externalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      externalsCall.setChildren([self.Cxx.getGroup(externalsInd)])
    extFuncInit   = ASE.Compiler.Cxx.Initializer.Initializer()
    extFuncInit.setChildren([externalsCall])
    extFuncDecl   = self.Cxx.getDeclaration(extFuncVar, self.IOR.getExternalType(type), extFuncInit)
    dimenFuncPtr  = ASE.Compiler.Cxx.StructReference.StructReference()
    dimenFuncPtr.setChildren([extFuncVar])
    dimenFuncPtr.setMember('getDimen')
    dimenFuncPtr.setPointer(1)
    dimenFunc     = ASE.Compiler.Cxx.Indirection.Indirection()
    dimenFunc.setChildren([dimenFuncPtr])
    arrayDimen    = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    arrayDimen.setChildren([self.Cxx.getGroup(dimenFunc)])
    arrayDimen.setArguments([arrayVar])
    arrayDimen.addArguments([self.Cxx.getNull()])
    dimenInit     = ASE.Compiler.Cxx.Initializer.Initializer()
    dimenInit.setChildren([arrayDimen])
    dimenDecl     = self.Cxx.getDeclaration(dimenVar, self.typeMap['int'], dimenInit)
    intDecl       = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.IOR.typeMap['int'])
    sizeofInt     = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    safeDimen     = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    lowerSize     = ASE.Compiler.Cxx.Multiplication.Multiplication()
    lowerSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    lowerMalloc   = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    lowerMalloc.setChildren([self.Cxx.getVar('malloc')])
    lowerMalloc.setArguments([lowerSize])
    indicesInit   = ASE.Compiler.Cxx.Initializer.Initializer()
    indicesInit.setChildren([self.Cxx.getInteger(0)])
    indicesInit.setList(1)
    indLowerVar   = self.Cxx.getVar('indLower')
    indicesLower  = ASE.Compiler.Cxx.Array.Array()
    indicesLower.setChildren([indLowerVar])
    indicesLower.setType(self.IOR.typeMap['int'])
    indicesLower.setSize(self.Cxx.getInteger(1))
    indicesLower.setInitializer(indicesInit)
    indUpperVar   = self.Cxx.getVar('indUpper')
    indicesUpper  = ASE.Compiler.Cxx.Array.Array()
    indicesUpper.setChildren([indUpperVar])
    indicesUpper.setType(self.IOR.typeMap['int'])
    indicesUpper.setSize(self.Cxx.getInteger(1))
    indicesUpper.setInitializer(indicesInit)
    indStrideVar  = self.Cxx.getVar('indStride')
    indicesStride = ASE.Compiler.Cxx.Array.Array()
    indicesStride.setChildren([indStrideVar])
    indicesStride.setType(self.IOR.typeMap['int'])
    indicesStride.setSize(self.Cxx.getInteger(1))
    indicesStride.setInitializer(indicesInit)
    lowerInit     = ASE.Compiler.Cxx.Initializer.Initializer()
    lowerInit.setChildren([lowerMalloc, indLowerVar, indUpperVar, indStrideVar, self.Cxx.getInteger(0), self.Cxx.getInteger(1)])
    lowerDecl     = self.Cxx.getDeclaration(lowerVar, self.IOR.getType(intArray).getChildren()[0], lowerInit)
    upperDecl     = self.Cxx.getDeclaration(upperVar, self.IOR.getType(intArray).getChildren()[0], lowerInit)
    numElemDecl   = self.Cxx.getDeclaration(numElemVar, self.IOR.getType(intArray).getChildren()[0], lowerInit)
    iInit         = ASE.Compiler.Cxx.Assignment.Assignment()
    iInit.setChildren([iVar, self.Cxx.getInteger(0)])
    iCheck        = ASE.Compiler.Cxx.LessThan.LessThan()
    iCheck.setChildren([iVar, dimenVar])
    iInc          = ASE.Compiler.Cxx.Increment.Increment()
    iInc.setChildren([iVar])
    iInc.setPostfix(1)
    lowerFirst    = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    lowerIndex    = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    lowerIndex.setChildren([lowerFirst])
    lowerIndex.setIndex(iVar)
    lowerFuncPtr  = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFuncPtr.setChildren([extFuncVar])
    lowerFuncPtr.setMember('getLower')
    lowerFuncPtr.setPointer(1)
    lowerFunc     = ASE.Compiler.Cxx.Indirection.Indirection()
    lowerFunc.setChildren([lowerFuncPtr])
    lowerCall     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    lowerCall.setChildren([self.Cxx.getGroup(lowerFunc)])
    lowerCall.setArguments([arrayVar, iVar])
    lowerCall.addArguments([self.Cxx.getNull()])
    setLower      = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([lowerIndex, lowerCall])
    upperFirst    = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    upperIndex    = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    upperIndex.setChildren([upperFirst])
    upperIndex.setIndex(iVar)
    upperFuncPtr  = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFuncPtr.setChildren([extFuncVar])
    upperFuncPtr.setMember('getUpper')
    upperFuncPtr.setPointer(1)
    upperFunc     = ASE.Compiler.Cxx.Indirection.Indirection()
    upperFunc.setChildren([upperFuncPtr])
    upperCall     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    upperCall.setChildren([self.Cxx.getGroup(upperFunc)])
    upperCall.setArguments([arrayVar, iVar])
    upperCall.addArguments([self.Cxx.getNull()])
    setUpper      = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([upperIndex, upperCall])
    numElemFirst  = ASE.Compiler.Cxx.StructReference.StructReference()
    numElemFirst.setChildren([numElemVar])
    numElemFirst.setMember('d_firstElement')
    numElemIndex  = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    numElemIndex.setChildren([numElemFirst])
    numElemIndex.setIndex(iVar)
    indexDiff     = ASE.Compiler.Cxx.Subtraction.Subtraction()
    indexDiff.setChildren([upperIndex, lowerIndex])
    dimSize       = ASE.Compiler.Cxx.Addition.Addition()
    dimSize.setChildren([indexDiff, self.Cxx.getInteger(1)])
    setNumElem    = ASE.Compiler.Cxx.Assignment.Assignment()
    setNumElem.setChildren([numElemIndex, dimSize])
    compound      = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren(map(self.Cxx.getExpStmt, [setLower, setUpper, setNumElem]))
    setDims       = ASE.Compiler.Cxx.For.For()
    setDims.setInitialization(iInit)
    setDims.setBranch(iCheck)
    setDims.setIncrement(iInc)
    setDims.setChildren([compound])
    pyArrayCreate = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    pyArrayCreate.setChildren([self.Cxx.getVar('PyArray_FromDims')])
    pyArrayCreate.setArguments([dimenVar, numElemFirst, self.getNumericTypeCode(type)])
    setReturn     = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([returnVar, self.Cxx.castToType(pyArrayCreate, self.getArrayType())])
    sizeVar       = self.Cxx.getVar('size')
    sizeInit      = ASE.Compiler.Cxx.Initializer.Initializer()
    sizeInit.setChildren([self.Cxx.getInteger(1)])
    dataVar       = self.Cxx.getVar('data')
    returnData    = ASE.Compiler.Cxx.StructReference.StructReference()
    returnData.setChildren([returnVar])
    returnData.setMember('data')
    returnData.setPointer(1)
    dataInit       = ASE.Compiler.Cxx.Initializer.Initializer()
    dataInit.setChildren([returnData])
    initIndex      = ASE.Compiler.Cxx.Assignment.Assignment()
    initIndex.setChildren([iVar, self.Cxx.getInteger(0)])
    checkIndex     = ASE.Compiler.Cxx.LessThan.LessThan()
    checkIndex.setChildren([iVar, dimenVar])
    incIndex       = ASE.Compiler.Cxx.Increment.Increment()
    incIndex.setChildren([iVar])
    incIndex.setPostfix(1)
    multSize      = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
    multSize.setChildren([sizeVar, numElemIndex])
    compound      = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(multSize)])
    calcSize      = ASE.Compiler.Cxx.For.For()
    calcSize.setInitialization(initIndex)
    calcSize.setBranch(checkIndex)
    calcSize.setIncrement(incIndex)
    calcSize.setChildren([compound])
    decSize       = ASE.Compiler.Cxx.Increment.Increment()
    decSize.setChildren([sizeVar])
    decSize.setDecrement(1)
    decSize.setPostfix(1)
    elemVar       = self.Cxx.getVar('elem')
    getElemFuncPtr= ASE.Compiler.Cxx.StructReference.StructReference()
    getElemFuncPtr.setChildren([extFuncVar])
    getElemFuncPtr.setMember('getElement')
    getElemFuncPtr.setPointer(1)
    getElemFunc   = ASE.Compiler.Cxx.Indirection.Indirection()
    getElemFunc.setChildren([getElemFuncPtr])
    getElement    = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getElement.setChildren([self.Cxx.getGroup(getElemFunc)])
    getElement.setArguments([arrayVar, self.Cxx.getAddress(lowerVar)])
    getElement.addArguments([self.Cxx.getNull()])
    elemInit      = ASE.Compiler.Cxx.Initializer.Initializer()
    elemInit.setChildren([getElement])
    compound      = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(elemVar, self.IOR.getType(type), elemInit)])
    if type.getBaseType():
      typeName = type.getIdentifier()
      if typeName == 'string':
        dataInd     = ASE.Compiler.Cxx.Indirection.Indirection()
        dataInd.setChildren([self.Cxx.castToType(dataVar, self.getObjectType(), 1)])
        convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        convertCall.setChildren([self.Cxx.getVar('PyString_FromString')])
        convertCall.setArguments([elemVar])
        setData     = ASE.Compiler.Cxx.Assignment.Assignment()
        setData.setChildren([dataInd, convertCall])
        freeElem    = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        freeElem.setChildren([self.Cxx.getVar('free')])
        freeElem.setArguments([self.Cxx.castToNamedType(elemVar, 'char', 1)])
        trueBranch  = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()        
        trueBranch.addChildren([self.Cxx.getExpStmt(setData), self.Cxx.getExpStmt(freeElem)])
        noneVar     = self.Cxx.getVar('Py_None')
        increfNone  = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        increfNone.setChildren([self.Cxx.getVar('Py_INCREF')])
        increfNone.setArguments([noneVar])
        setData     = ASE.Compiler.Cxx.Assignment.Assignment()
        setData.setChildren([dataInd, noneVar])
        falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()        
        falseBranch.addChildren([self.Cxx.getExpStmt(increfNone), self.Cxx.getExpStmt(setData)])
        checkElem   = ASE.Compiler.Cxx.If.If()
        checkElem.setBranch(elemVar)
        checkElem.setChildren([trueBranch, falseBranch])
        compound.addChildren([checkElem])
      elif typeName == 'opaque':
        dataInd     = ASE.Compiler.Cxx.Indirection.Indirection()
        dataInd.setChildren([self.Cxx.castToType(dataVar, self.getObjectType(), 1)])
        convertCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        convertCall.setChildren([self.Cxx.getVar('PyCObject_FromVoidPtr')])
        convertCall.setArguments([elemVar, self.Cxx.getNull()])
        setData     = ASE.Compiler.Cxx.Assignment.Assignment()
        setData.setChildren([dataInd, convertCall])
        compound.addChildren([self.Cxx.getExpStmt(setData)])
      elif typeName in ['dcomplex', 'fcomplex']:
        dataInd     = ASE.Compiler.Cxx.Indirection.Indirection()
        dataInd.setChildren([self.Cxx.castToType(dataVar, self.IOR.getType(type), 1)])
        self.setValue_dcomplex(elemVar, dataInd, compound, setIOR = 0, pythonImagName = 'imaginary')
      else:
        dataInd     = ASE.Compiler.Cxx.Indirection.Indirection()
        dataInd.setChildren([self.Cxx.castToType(dataVar, self.getType(type), 1)])
        setData     = ASE.Compiler.Cxx.Assignment.Assignment()
        setData.setChildren([dataInd, elemVar])
        compound.addChildren([self.Cxx.getExpStmt(setData)])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      trueIor = ASE.Compiler.Cxx.StructReference.StructReference()
      trueIor.setChildren([elemVar])
      trueIor.setMember('d_object')
      trueIor.setPointer(1)
      elemEpv = ASE.Compiler.Cxx.StructReference.StructReference()
      elemEpv.setChildren([elemVar])
      elemEpv.setMember('d_epv')
      elemEpv.setPointer(1)
      increfFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      increfFuncPtr.setChildren([elemEpv])
      increfFuncPtr.setMember('addReference')
      increfFuncPtr.setPointer(1)
      increfFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      increfFunc.setChildren([self.Cxx.getGroup(increfFuncPtr)])
      increfCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      increfCall.setChildren([self.Cxx.getGroup(increfFunc)])
      increfCall.setArguments([trueIor, self.Cxx.getNull()])
      dataInd     = ASE.Compiler.Cxx.Indirection.Indirection()
      dataInd.setChildren([self.Cxx.castToType(dataVar, self.getObjectType(), 1)])
      getData     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      getData.setChildren([self.getWrapIOR(type).getChildren()[0]])
      getData.setArguments([elemVar])
      setData     = ASE.Compiler.Cxx.Assignment.Assignment()
      setData.setChildren([dataInd, getData])
      negDataInd  = ASE.Compiler.Cxx.Negation.Negation()
      negDataInd.setChildren([self.Cxx.getGroup(dataInd)])
      checkData   = ASE.Compiler.Cxx.If.If()
      decrefReturn = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefReturn.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefReturn.setArguments([returnVar])
      resetReturn = ASE.Compiler.Cxx.Assignment.Assignment()
      resetReturn.setChildren([returnVar, self.Cxx.getNull()])
      gotoError   = ASE.Compiler.Cxx.Goto.Goto()
      gotoError.setIdentifier('error')
      cmpd        = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      cmpd.setChildren([self.Cxx.getExpStmt(decrefReturn), self.Cxx.getExpStmt(resetReturn), gotoError])
      checkData.setBranch(negDataInd)
      checkData.setChildren([cmpd])
      compound.addChildren([self.Cxx.getExpStmt(increfCall), self.Cxx.getExpStmt(setData), checkData])
    returnStride  = ASE.Compiler.Cxx.StructReference.StructReference()
    returnStride.setChildren([returnVar])
    returnStride.setMember('strides')
    returnStride.setPointer(1)
    strideIndex   = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    strideIndex.setChildren([returnStride])
    strideIndex.setIndex(iVar)
    incLowerIndex = ASE.Compiler.Cxx.Increment.Increment()
    incLowerIndex.setChildren([self.Cxx.getGroup(lowerIndex)])
    cmpLower      = ASE.Compiler.Cxx.LessThanOrEqual.LessThanOrEqual()
    cmpLower.setChildren([incLowerIndex, upperIndex])
    incData       = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
    incData.setChildren([dataVar, strideIndex])
    trueBranch    = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([ASE.Compiler.Cxx.Break.Break()])
    sizeIndex     = ASE.Compiler.Cxx.Multiplication.Multiplication()
    sizeIndex.setChildren([strideIndex, numElemIndex])
    decData       = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
    decData.setChildren([dataVar, sizeIndex])
    decLower      = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
    decLower.setChildren([lowerIndex, numElemIndex])
    falseBranch   = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(decData), self.Cxx.getExpStmt(decLower)])
    checkLower    = ASE.Compiler.Cxx.If.If()
    checkLower.setBranch(cmpLower)
    checkLower.setChildren([trueBranch, falseBranch])
    cmpd          = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    cmpd.setChildren([self.Cxx.getExpStmt(incData), checkLower])
    incLower      = ASE.Compiler.Cxx.For.For()
    incLower.setInitialization(iInit)
    incLower.setBranch(iCheck)
    incLower.setIncrement(iInc)
    incLower.setChildren([cmpd])
    compound.addChildren([incLower])
    sizeLoop      = ASE.Compiler.Cxx.While.While()
    sizeLoop.setBranch(decSize)
    sizeLoop.setChildren([compound])
    compound      = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(sizeVar, self.typeMap['int'], sizeInit), self.Cxx.getDeclaration(dataVar, charPtr, dataInit)])
    compound.setChildren([calcSize, sizeLoop])
    checkReturn   = ASE.Compiler.Cxx.If.If()
    checkReturn.setBranch(arrayVar)
    checkReturn.setChildren([compound])
    freeVar       = self.Cxx.getVar('free')
    freeLower     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeLower.setChildren([freeVar])
    freeLower.setArguments([lowerFirst])
    freeLower     = self.Cxx.getExpStmt(freeLower)
    freeLower.setIdentifier('error')
    freeUpper     = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeUpper.setChildren([freeVar])
    freeUpper.setArguments([upperFirst])
    freeNumElem   = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeNumElem.setChildren([freeVar])
    freeNumElem.setArguments([numElemFirst])
    mainBody.addDeclarations([extFuncDecl,
                              self.Cxx.getDecl(indicesLower), self.Cxx.getDecl(indicesUpper), self.Cxx.getDecl(indicesStride),
                              dimenDecl, lowerDecl, upperDecl, numElemDecl,
                              self.Cxx.getDeclaration(iVar, self.typeMap['int'])])
    mainBody.setChildren([setDims, self.Cxx.getExpStmt(setReturn), checkReturn, freeLower]+map(self.Cxx.getExpStmt, [freeUpper, freeNumElem]))
    checkArray   = ASE.Compiler.Cxx.If.If()
    checkArray.setBranch(arrayVar)
    checkArray.setChildren([mainBody])
    returnStmt   = ASE.Compiler.Cxx.Return.Return()
    returnStmt.addChildren([self.Cxx.castToType(returnVar, self.getObjectType())])
    body.addChildren([checkArray, returnStmt])

    arrayParam   = ASE.Compiler.Cxx.Parameter.Parameter()
    arrayParam.setIdentifier(arrayVar.getIdentifier())
    arrayParam.setType(self.IOR.getType(arrayType))
    wrapName     = ASE.Compiler.Cxx.Declarator.Declarator()
    wrapName.setIdentifier(type.getFullIdentifier().replace('.', '_')+'__wrap_array')
    wrapFunc     = ASE.Compiler.Cxx.Function.Function()
    wrapFunc.setChildren([wrapName])
    wrapFunc.setType(self.getObjectType())
    wrapFunc.setStatic(1)
    wrapFunc.setParameters([arrayParam])
    wrapFunc.setBody(body)
    self.wrapArray[type] = wrapFunc
    return wrapFunc
  
  def parseFormat_bool(self, parameter):
    return 'i'
  
  def parseArgument_bool(self, parameter, isArgument):
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [var]
  
  def parseConvert_bool(self, parameter, body, isArgument):
    if not isArgument:
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    return
  
  def buildArgument_bool(self, parameter, isArgument):
    return [self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvert_bool(self, parameter, body, isArgument):
    if not isArgument and not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    return
  
  def parseFormat_char(self, parameter):
    return 'c'
  
  def parseArgument_char(self, parameter, isArgument):
    return self.parseArgument_bool(parameter, isArgument)
  
  def parseConvert_char(self, parameter, body, isArgument):
    return self.parseConvert_bool(parameter, body, isArgument)
  
  def buildArgument_char(self, parameter, isArgument):
    return self.buildArgument_bool(parameter, isArgument)
  
  def buildConvert_char(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def parseFormat_dcomplex(self, parameter):
    return 'D'
  
  def setValue_dcomplex(self, iorVar, pythonVar, body, setIOR, pythonImagName = 'imag'):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.StructReference
    iorReal = ASE.Compiler.Cxx.StructReference.StructReference()
    iorReal.setChildren([self.Cxx.getGroup(iorVar)])
    iorReal.setMember('real')
    pythonReal = ASE.Compiler.Cxx.StructReference.StructReference()
    pythonReal.setChildren([self.Cxx.getGroup(pythonVar)])
    pythonReal.setMember('real')
    iorImag = ASE.Compiler.Cxx.StructReference.StructReference()
    iorImag.setChildren([self.Cxx.getGroup(iorVar)])
    iorImag.setMember('imaginary')
    pythonImag = ASE.Compiler.Cxx.StructReference.StructReference()
    pythonImag.setChildren([self.Cxx.getGroup(pythonVar)])
    pythonImag.setMember(pythonImagName)
    if setIOR:
      setReal = ASE.Compiler.Cxx.Assignment.Assignment()
      setReal.setChildren([iorReal, pythonReal])
      setImag = ASE.Compiler.Cxx.Assignment.Assignment()
      setImag.setChildren([iorImag, pythonImag])
    else:
      setReal = ASE.Compiler.Cxx.Assignment.Assignment()
      setReal.setChildren([pythonReal, iorReal])
      setImag = ASE.Compiler.Cxx.Assignment.Assignment()
      setImag.setChildren([pythonImag, iorImag])
    body.addChildren([self.Cxx.getDecl(setReal), self.Cxx.getDecl(setImag)])
    return
  
  def parseArgument_dcomplex(self, parameter, isArgument):
    return [self.Cxx.getAddress(self.Cxx.getVar('_local_' + parameter.getIdentifier()))]
  
  def parseConvert_dcomplex(self, parameter, body, isArgument):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.StructReference
    iorVar = self.getParseConvertVar(parameter, isArgument)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    if not isArgument:
      if iorVar.isInstanceOf('ASE.Compiler.Cxx.Variable'):
        body.addDeclarations([self.Cxx.getDeclaration(iorVar, self.IOR.getType(parameter.getType()))])
    if not isArgument or not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, self.getType(parameter.getType()))])
    self.setValue_dcomplex(iorVar, pythonVar, body, setIOR = 1)
    return
  
  def buildArgument_dcomplex(self, parameter, isArgument):
    return [self.Cxx.getAddress(self.Cxx.getVar('_local_' + parameter.getIdentifier()))]
  
  def buildConvert_dcomplex(self, parameter, body, isArgument):
    iorVar = self.getBuildConvertVar(parameter, isArgument)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    if not isArgument and not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    if isArgument or not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, self.getType(parameter.getType()))])
    self.setValue_dcomplex(iorVar, pythonVar, body, setIOR = 0)
    return
  
  def parseFormat_double(self, parameter):
    return 'd'
  
  def parseArgument_double(self, parameter, isArgument):
    return self.parseArgument_bool(parameter, isArgument)
  
  def parseConvert_double(self, parameter, body, isArgument):
    return self.parseConvert_bool(parameter, body, isArgument)
  
  def buildArgument_double(self, parameter, isArgument):
    return self.buildArgument_bool(parameter, isArgument)
  
  def buildConvert_double(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def parseFormat_fcomplex(self, parameter):
    return 'D'
  
  def parseArgument_fcomplex(self, parameter, isArgument):
    return self.parseArgument_dcomplex(parameter, isArgument)
  
  def parseConvert_fcomplex(self, parameter, body, isArgument):
    return self.parseConvert_dcomplex(parameter, body, isArgument)
  
  def buildArgument_fcomplex(self, parameter, isArgument):
    return self.buildArgument_dcomplex(parameter, isArgument)
  
  def buildConvert_fcomplex(self, parameter, body, isArgument):
    return self.buildConvert_dcomplex(parameter, body, isArgument)
  
  def parseFormat_float(self, parameter):
    return 'f'
  
  def parseArgument_float(self, parameter, isArgument):
    return self.parseArgument_bool(parameter, isArgument)
  
  def parseConvert_float(self, parameter, body, isArgument):
    return self.parseConvert_bool(parameter, body, isArgument)
  
  def buildArgument_float(self, parameter, isArgument):
    return self.buildArgument_bool(parameter, isArgument)
  
  def buildConvert_float(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def parseFormat_int(self, parameter):
    return 'l'
  
  def parseArgument_int(self, parameter, isArgument):
    return self.parseArgument_dcomplex(parameter, isArgument)
  
  def parseConvert_int(self, parameter, body, isArgument):
    import ASE.Compiler.Cxx.Assignment
    iorVar = self.getParseConvertVar(parameter, isArgument)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    if not isArgument:
      if iorVar.isInstanceOf('ASE.Compiler.Cxx.Variable'):
        body.addDeclarations([self.Cxx.getDeclaration(iorVar, self.IOR.getType(parameter.getType()))])
    if not isArgument or not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, self.getType(parameter.getType()))])
    setIorVar = ASE.Compiler.Cxx.Assignment.Assignment()
    setIorVar.setChildren([iorVar, pythonVar])
    body.addChildren([self.Cxx.getExpStmt(setIorVar)])
    return
  
  def buildArgument_int(self, parameter, isArgument):
    return [self.Cxx.getVar('_local_' + parameter.getIdentifier())]
  
  def buildConvert_int(self, parameter, body, isArgument):
    import ASE.Compiler.Cxx.Assignment
    type = parameter.getType()
    iorVar = self.getBuildConvertVar(parameter, isArgument)
    pythonVar = self.Cxx.getVar('_local_' + parameter.getIdentifier())
    if not isArgument and not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    if isArgument or not self.isIn(parameter):
      body.addDeclarations([self.Cxx.getDeclaration(pythonVar, self.getType(type))])
    setIOR = ASE.Compiler.Cxx.Assignment.Assignment()
    setIOR.setChildren([pythonVar, iorVar])
    body.addChildren([self.Cxx.getExpStmt(setIOR, 'Convert ' + self.IOR.getType(type).getIdentifier() + ' to ' + self.getType(type).getIdentifier())])
    return
  
  def parseFormat_long(self, parameter):
    return 'L'
  
  def parseArgument_long(self, parameter, isArgument):
    return self.parseArgument_dcomplex(parameter, isArgument)
  
  def parseConvert_long(self, parameter, body, isArgument):
    return self.parseConvert_int(parameter, body, isArgument)
  
  def buildArgument_long(self, parameter, isArgument):
    return self.buildArgument_int(parameter, isArgument)
  
  def buildConvert_long(self, parameter, body, isArgument):
    return self.buildConvert_int(parameter, body, isArgument)
  
  def parseFormat_opaque(self, parameter):
    return 'O&'
  
  def parseArgument_opaque(self, parameter, isArgument):
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractOpaque(parameter.getType()).getChildren()[0]), var]
  
  def parseConvert_opaque(self, parameter, body, isArgument):
    return self.parseConvert_bool(parameter, body, isArgument)
  
  def buildArgument_opaque(self, parameter, isArgument):
    return [self.Cxx.castToVoidPointer(self.getWrapOpaque(parameter.getType()).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvert_opaque(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def copyValue_string(self, var, body):
    import ASE.Compiler.Cxx.Addition
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    strLen = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strLen.setChildren([self.Cxx.getVar('strlen')])
    strLen.setArguments([var])
    strSize = ASE.Compiler.Cxx.Addition.Addition()
    strSize.setChildren([strLen, self.Cxx.getInteger(1)])
    strAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strAlloc.setChildren([self.Cxx.getVar('malloc')])
    strAlloc.setArguments([strSize])
    strCopy = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strCopy.setChildren([self.Cxx.getVar('strcpy')])
    strCopy.setArguments([strAlloc, var])
    strCheck = ASE.Compiler.Cxx.If.If()
    strCheck.setBranch(var)
    strCheck.setChildren([strCopy, self.Cxx.getNull()])
    strCheck.setTernary(1)
    strSet = ASE.Compiler.Cxx.Assignment.Assignment()
    strSet.setChildren([var, strCheck])
    body.addChildren([self.Cxx.getExpStmt(strSet, 'Copying the Python string to IOR controlled memory')])
    return
  
  def parseFormat_string(self, parameter):
    return 'z'
  
  def parseArgument_string(self, parameter, isArgument):
    return self.parseArgument_bool(parameter, isArgument)
  
  def parseConvert_string(self, parameter, body, isArgument):
    if not isArgument:
      body.addDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar(parameter.getIdentifier()), self.IOR.getType(parameter.getType()))])
    if not isArgument or self.isOut(parameter):
      self.copyValue_string(self.getParseConvertVar(parameter, isArgument), body)
    return
  
  def buildArgument_string(self, parameter, isArgument):
    return self.buildArgument_bool(parameter, isArgument)
  
  def buildConvert_string(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def buildCleanup_string(self, parameter, body, isArgument):
    '''
    Should free string memory after a build call, unless the original string can be referenced
    '''
    if not isArgument:
      import ASE.Compiler.Cxx.FunctionCall
      var = self.getBuildConvertVar(parameter, isArgument)
      freeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      freeCall.setChildren([self.Cxx.getVar('free')])
      freeCall.setArguments([self.Cxx.castToVoidPointer(var)])
      body.addChildren([self.Cxx.getExpStmt(freeCall, 'Freeing IOR memory since Python has made a copy of the string')])
    return
  
  def parseFormatEnumeration(self, parameter):
    return self.parseFormat_int(parameter)
  
  def parseArgumentEnumeration(self, parameter, isArgument):
    return self.parseArgument_dcomplex(parameter, isArgument)
  
  def parseConvertEnumeration(self, parameter, body, isArgument):
    return self.parseConvert_int(parameter, body, isArgument)
  
  def buildArgumentEnumeration(self, parameter, isArgument):
    return self.buildArgument_int(parameter, isArgument)
  
  def buildConvertEnumeration(self, parameter, body, isArgument):
    return self.buildConvert_int(parameter, body, isArgument)
  
  def buildCleanupEnumeration(self, parameter, body, isArgument):
    return
  
  def parseFormatArray(self, parameter):
    return self.parseFormat_opaque(parameter)
  
  def parseArgumentArray(self, parameter, isArgument):
    import ASE.Compiler.SIDL.Array
    entryType = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractArray(entryType, borrow = not self.isOut(parameter)).getChildren()[0]), var]
  
  def parseConvertArray(self, parameter, body, isArgument):
    return self.parseConvert_bool(parameter, body, isArgument)
  
  def buildArgumentArray(self, parameter, isArgument):
    import ASE.Compiler.SIDL.Array
    entryType = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
    return [self.Cxx.castToVoidPointer(self.getWrapArray(entryType).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvertArray(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def buildCleanupArray(self, parameter, body, isArgument):
    return
  
  def parseFormatInterface(self, parameter):
    return self.parseFormat_opaque(parameter)
  
  def parseArgumentInterface(self, parameter, isArgument):
    if isArgument:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = self.Cxx.getAddress(self.Cxx.getVar(parameter.getIdentifier()))
    return [self.Cxx.castToVoidPointer(self.getExtractIOR(parameter.getType()).getChildren()[0]), var]
  
  def parseConvertInterface(self, parameter, body, isArgument):
    return self.parseConvert_bool(parameter, body, isArgument)
  
  def buildArgumentInterface(self, parameter, isArgument, isWeak = 0):
    return [self.Cxx.castToVoidPointer(self.getWrapIOR(parameter.getType(), isWeak = isWeak).getChildren()[0]), self.getBuildConvertVar(parameter, isArgument)]
  
  def buildConvertInterface(self, parameter, body, isArgument):
    return self.buildConvert_bool(parameter, body, isArgument)
  
  def buildCleanupInterface(self, parameter, body, isArgument):
    return
  
  def getParseFormat(self, parameter):
    '''
    Get the Python format string corresponding to a SIDL type
    '''
    type = parameter.getType()
    funcName = 'parseFormat_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      return getattr(self, funcName)(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      return self.parseFormatEnumeration(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      return self.parseFormatArray(parameter)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      return self.parseFormatInterface(parameter)
    return ''
  
  def getParseArguments(self, parameter, isArgument):
    '''
    Return a list of arguments to be passed to PyArg_Parse* functions for the given parameter
    '''
    type = parameter.getType()
    funcName = 'parseArgument_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      params = getattr(self, funcName)(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      params = self.parseArgumentEnumeration(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      params = self.parseArgumentArray(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      params = self.parseArgumentInterface(parameter, isArgument)
    else:
      params = []
    return params
  
  def getParseConvertVar(self, parameter, isArgument):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Indirection
    if not isArgument or parameter.getAccessMode() == ASE.Compiler.ParameterAccessMode.RETURN:
      var = self.Cxx.getVar(parameter.getIdentifier())
    else:
      var = ASE.Compiler.Cxx.Indirection.Indirection()
      var.setChildren([self.Cxx.getVar(parameter.getIdentifier())])
    return var
  
  def getConvertParseArgument(self, parameter, body, isArgument):
    '''
    Here we convert from local Python parameters to IOR parameters
       - Declaration of both parameters is also done here
    '''
    type = parameter.getType()
    funcName = 'parseConvert_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      getattr(self, funcName)(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      self.parseConvertEnumeration(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.parseConvertArray(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.parseConvertInterface(parameter, body, isArgument)
    return
  
  def getBuildArguments(self, parameter, isArgument):
    '''
    Convert IOR out parameters to types needed by Py_BuildValue() and construct the parameter list
       - Local variables (_local_foo) are of the type needed by Python
    '''
    type = parameter.getType()
    funcName = 'buildArgument_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      params = getattr(self, funcName)(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      params = self.buildArgumentEnumeration(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      params = self.buildArgumentArray(parameter, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      params = self.buildArgumentInterface(parameter, isArgument)
    else:
      params = []
    return params
  
  def getBuildConvertVar(self, parameter, isArgument):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Indirection
    if isArgument and self.isOut(parameter):
      var = ASE.Compiler.Cxx.Indirection.Indirection()
      var.setChildren([self.Cxx.getVar(parameter.getIdentifier())])
    else:
      var = self.Cxx.getVar(parameter.getIdentifier())
    return var
  
  def getConvertBuildParameter(self, parameter, body, isArgument):
    type = parameter.getType()
    funcName = 'buildConvert_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      getattr(self, funcName)(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      self.buildConvertEnumeration(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.buildConvertArray(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.buildConvertInterface(parameter, body, isArgument)
    return
  
  def getCleanupBuildParameter(self, parameter, body, isArgument):
    type = parameter.getType()
    funcName = 'buildCleanup_' + str(type.getIdentifier())
    if hasattr(self, funcName):
      getattr(self, funcName)(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      self.buildCleanupEnumeration(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      self.buildCleanupArray(parameter, body, isArgument)
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.buildCleanupInterface(parameter, body, isArgument)
    return
