#!/usr/bin/env python

# ----------------------------------------------------------------------
# sidllex.py
#
# A lexer for SIDL.
# ----------------------------------------------------------------------

import lex

# Reserved words
reserved = (
  'ARRAY', 'BOOL', 'CHAR', 'CLASS', 'DCOMPLEX', 'DOUBLE', 'ENUM', 'EXTENDS', 'FCOMPLEX', 'FLOAT', 'IN', 'INT', 'INOUT',
  'IMPLEMENTS', 'INTERFACE', 'LONG', 'OPAQUE', 'OUT', 'PACKAGE', 'STATIC', 'STRING', 'THROWS', 'UNIVERSE', 'VOID'
  )

tokens = reserved + (
  # Literals (scoped identifier, identifier, integer constant, float constant, string constant, char const)
  'SCOPEDID', 'ID', 'ICONST',# 'FCONST', 'SCONST', 'CCONST', 

  # Assignment
  'EQUALS',

  # Delimeters < > ( ) [ ] { } , . ; :
  'LT', 'GT',
  'LPAREN', 'RPAREN',
  'LBRACKET', 'RBRACKET',
  'LBRACE', 'RBRACE',
  'COMMA', 'SEMICOLON', 'COLON',

  # Comments
  'COMMENT'
  )

# Newlines
def t_NEWLINE(t):
  r'\n+'
  t.lineno += t.value.count('\n')

# Completely ignored characters
t_ignore    = ' \t\x0c'
# Assignment
t_EQUALS    = r'='
# Delimeters
t_LT        = r'<'
t_GT        = r'>'
t_LPAREN    = r'\('
t_RPAREN    = r'\)'
t_LBRACKET  = r'\['
t_RBRACKET  = r'\]'
t_LBRACE    = r'\{'
t_RBRACE    = r'\}'
t_COMMA     = r','
t_SEMICOLON = r';'
t_COLON    = r':'
#t_ELLIPSIS = r'\.\.\.'

# Identifiers and reserved words
reserved_map = { }
for r in reserved:
    reserved_map[r.lower()] = r

def t_SCOPEDID(t):
    r'[A-Za-z_][\w_-]*(\.[A-Za-z_][\w_-]*)+'
    t.type = reserved_map.get(t.value, 'SCOPEDID')
    return t

def t_ID(t):
    r'[A-Za-z_][\w_-]*'
    t.type = reserved_map.get(t.value, 'ID')
    return t

# Integer literal
t_ICONST = r'[+-]?\d+([uU]|[lL]|[uU][lL]|[lL][uU])?'

# Floating literal
#t_FCONST = r'((\d+)(\.\d+)(e(\+|-)?(\d+))? | (\d+)e(\+|-)?(\d+))([lL]|[fF])?'

# String literal
#t_SCONST = r'\"([^\\\n]|(\\.))*?\"'

# Character constant 'c' or L'c'
#t_CCONST = r'(L)?\'([^\\\n]|(\\.))*?\''

# Comments 
def t_COMMENT(t):
  r'/\*(.|\n)*?\*/|//(.*)|/\*[^*]*\*+([^/*][^*]*\*+)*/|"(\\.|[^"\\])*"'
  return t
    
def t_error(t):
  print "Illegal character %s [line %d]" % (repr(t.value[0]), t.lineno)
  t.skip(1)

lexer = lex.lex(optimize = 1)
if __name__ == "__main__":
  lex.runmain(lexer)
