#!/usr/bin/env python

# -----------------------------------------------------------------------------
# sidlparse.py
#
# Simple parser for SIDL
# -----------------------------------------------------------------------------
import user
import importer
import ASE.Compiler.ParameterAccessMode
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.Comment
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse

import yacc
import sidllex
import sys

# Get the token map
tokens    = sidllex.tokens
baseTypes = {}

# specification:
def p_specification_1(t):
  'specification : package_specifier_list'
  spec = ASE.Compiler.SIDL.Specification.Specification()
  spec.addChildren(t[1])
  t[0] = spec

def p_specification_empty(t):
  'specification : empty'
  spec = ASE.Compiler.SIDL.Specification.Specification()
  t[0] = spec

# package-specifier-list
def p_package_specifier_list_1(t):
  'package_specifier_list : package_specifier_list package_specifier'
  t[0] = t[1]+[t[2]]

def p_package_specifier_list_2(t):
  'package_specifier_list : package_specifier'
  t[0] = [t[1]]

# package-specifier
def p_package_specifier_1(t):
  'package_specifier : PACKAGE ID LBRACE definition_list RBRACE optional_semi'
  package = ASE.Compiler.SIDL.Package.Package()
  package.setIdentifier(t[2])
  if t[4]:
    package.addChildren(t[4])
  t[0] = package

def p_package_specifier_2(t):
  'package_specifier : PACKAGE ID LBRACKET ID RBRACKET LBRACE definition_list RBRACE optional_semi'
  package = ASE.Compiler.SIDL.Package.Package()
  package.setIdentifier(t[2])
  if t[7]:
    package.addChildren(t[7])
  package.setTemplateIdentifiers([t[4]])
  t[0] = package

def p_package_specifier_3(t):
  'package_specifier : comment_list package_specifier'
  t[2].addComments(t[1])
  t[0] = t[2]

# definition-list
def p_definition_list_1(t):
  'definition_list : package_specifier definition_list'
  t[0] = [t[1]]+t[2]

def p_definition_list_2(t):
  'definition_list : enum_specifier definition_list'
  t[0] = [t[1]]+t[2]

def p_definition_list_3(t):
  'definition_list : type_universe_specifier definition_list'
  t[0] = [t[1]]+t[2]

def p_definition_list_4(t):
  'definition_list : interface_specifier definition_list'
  t[0] = [t[1]]+t[2]

def p_definition_list_5(t):
  'definition_list : class_specifier definition_list'
  t[0] = [t[1]]+t[2]

def p_definition_list_empty(t):
  'definition_list : empty'
  t[0] = []

# enum-specifier:
def p_enum_specifier_1(t):
  'enum_specifier : ENUM ID LBRACE enumerator_list RBRACE optional_semi'
  enum = ASE.Compiler.SIDL.Enumeration.Enumeration()
  enum.setIdentifier(t[2])
  if t[4]:
    enum.addChildren(t[4])
  t[0] = enum

def p_enum_specifier_2(t):
  'enum_specifier : ENUM LBRACE enumerator_list RBRACE optional_semi'
  enum = ASE.Compiler.SIDL.Enumeration.Enumeration()
  if t[3]:
    enum.addChildren(t[3])
  t[0] = enum

def p_enum_specifier_3(t):
  'enum_specifier : comment_list enum_specifier'
  t[2].addComments(t[1])
  t[0] = t[2]

# enumerator_list:
def p_enumerator_list_1(t):
  'enumerator_list : enumerator'
  t[0] = [t[1]]

def p_enumerator_list_2(t):
  'enumerator_list : enumerator_list COMMA enumerator'
  t[0] = t[1]+[t[3]]

# enumerator:
def p_enumerator_1(t):
  'enumerator : ID'
  enum = ASE.Compiler.SIDL.Enumerator.Enumerator()
  enum.setIdentifier(t[1])
  t[0] = enum

def p_enumerator_2(t):
  '''enumerator : ID EQUALS ICONST'''
  enum = ASE.Compiler.SIDL.Enumerator.Enumerator()
  enum.setIdentifier(t[1])
  enum.setValue(int(t[3]))
  enum.setInitialized(1)
  t[0] = enum

# type-universe-specifier
def p_type_universe_specifier(t):
  'type_universe_specifier : UNIVERSE ID LBRACE type_list RBRACE optional_semi'
  universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse()
  universe.setIdentifier(t[2])
  universe.setTypes(t[4])
  t[0] = universe

# interface-specifier
def p_interface_specifier_1(t):
  'interface_specifier : INTERFACE ID extends_specifier LBRACE declaration_list RBRACE optional_semi'
  t[0] = createInterface(t[2], t[3], t[5])

def p_interface_specifier_2(t):
  'interface_specifier : INTERFACE ID LBRACKET ID RBRACKET extends_specifier LBRACE declaration_list RBRACE optional_semi'
  t[0] = createInterface(t[2], t[6], t[8], [t[4]])

def p_interface_specifier_3(t):
  'interface_specifier : comment_list interface_specifier'
  t[2].addComments(t[1])
  t[0] = t[2]

def createInterface(name, parents, methods, templateIdentifiers = None):
  interface = ASE.Compiler.SIDL.Interface.Interface()
  interface.setIdentifier(name)
  interface.setBaseType(0)
  if parents:
    interfaceObjs = []
    for name in parents:
      interf = ASE.Compiler.SIDL.Interface.Interface()
      interf.setParent(interface)
      interf.setIdentifier(name)
      interface.setBaseType(0)
      interfaceObjs.append(interf)
    interface.addParents(interfaceObjs)
  if methods:
    interface.addChildren(methods)
  if not templateIdentifiers is None:
    interface.setTemplateIdentifiers(templateIdentifiers)
  return interface

# extends-specifier
def p_extends_specifier(t):
  'extends_specifier : EXTENDS scoped_identifier_list'
  t[0] = t[2]

def p_extends_specifier_empty(t):
  'extends_specifier : empty'
  t[0] = []

# declaration-list
def p_declaration_list_1(t):
  'declaration_list : method declaration_list'
  t[0] = [t[1]]+t[2]

def p_declaration_list_2(t):
  'declaration_list : comment_list method declaration_list'
  t[2].addComments(t[1])
  t[0] = [t[2]]+t[3]

def p_declaration_list_empty(t):
  'declaration_list : empty'
  t[0] = []

# class-specifier
def p_class_specifier_1(t):
  'class_specifier : CLASS ID extends_specifier implements_specifier LBRACE class_declaration_list RBRACE optional_semi'
  t[0] = createClass(t[2], t[3], t[4], t[6])

def p_class_specifier_2(t):
  'class_specifier : CLASS ID LBRACKET ID RBRACKET extends_specifier implements_specifier LBRACE class_declaration_list RBRACE optional_semi'
  t[0] = createClass(t[2], t[6], t[7], t[9], [t[4]])

def p_class_specifier_3(t):
  'class_specifier : comment_list class_specifier'
  t[2].addComments(t[1])
  t[0] = t[2]

def createClass(name, parents, interfaces, methods, templateIdentifiers = None):
  sclass = ASE.Compiler.SIDL.Class.Class()
  sclass.setIdentifier(name)
  sclass.setBaseType(0)
  if parents:
    parentObjs = []
    for name in parents:
      interface = ASE.Compiler.SIDL.Class.Class()
      interface.setParent(sclass)
      interface.setIdentifier(name)
      interface.setBaseType(0)
      parentObjs.append(interface)
    sclass.addParents(parentObjs)
  if interfaces:
    interfaceObjs = []
    for name in interfaces:
      interface = ASE.Compiler.SIDL.Interface.Interface()
      interface.setParent(sclass)
      interface.setIdentifier(name)
      interface.setBaseType(0)
      interfaceObjs.append(interface)
    sclass.addInterfaces(interfaceObjs)
  if methods:
    sclass.addChildren(methods)
  if not templateIdentifiers is None:
    sclass.setTemplateIdentifiers(templateIdentifiers)
  return sclass

# implements-specifier
def p_implements_specifier_1(t):
  'implements_specifier : IMPLEMENTS scoped_identifier_list'
  t[0] = t[2]

def p_implements_specifier_empty(t):
  'implements_specifier : empty'
  t[0] = []

# class-declaration-list
def p_class_declaration_list_1(t):
  'class_declaration_list : class_method class_declaration_list'
  t[0] = [t[1]]+t[2]

def p_class_declaration_list_2(t):
  'class_declaration_list : comment_list method class_declaration_list'
  t[2].addComments(t[1])
  t[0] = [t[2]]+t[3]

def p_class_declaration_list_3(t):
  'class_declaration_list : method class_declaration_list'
  t[0] = [t[1]]+t[2]

def p_class_declaration_list_4(t):
  'class_declaration_list : comment_list class_method class_declaration_list'
  t[2].addComments(t[1])
  t[0] = [t[2]]+t[3]

def p_class_declaration_list_empty(t):
  'class_declaration_list : empty'
  t[0] = []

# class-method
def p_class_method(t):
  'class_method : STATIC method'
  t[2].setStatic(1)
  t[0] = t[2]

# method
def p_method_1(t):
  'method : return_parameter_specifier ID LPAREN parameter_list RPAREN throws_specifier SEMICOLON'
  t[0] = createMethod(t[1], t[2], t[4], t[6])
  return

def p_method_2(t):
  'method : return_parameter_specifier ID LBRACKET ID RBRACKET LPAREN parameter_list RPAREN throws_specifier SEMICOLON'
  t[0] = createMethod(t[1], t[2], t[7], t[9], [t[4]])
  return

def createMethod(returnParameter, name, parameters, exceptions, templateIdentifiers = None):
  from ASE.Compiler.CodePurpose import STUB, CARTILAGE, IOR

  method = ASE.Compiler.SIDL.Method.Method()
  if not returnParameter is None:
    method.setReturnParameter(returnParameter)
    returnParameter.setParent(method)
  method.setIdentifier(name)
  if parameters:
    method.addChildren(parameters)
  if exceptions:
    excs = []
    for id in exceptions:
      exception = ASE.Compiler.SIDL.Interface.Interface()
      exception.setParent(method)
      exception.setIdentifier(id)
      exception.setBaseType(0)
      excs.append(exception)
    method.addExceptions(excs)
  if not templateIdentifiers is None:
    method.setTemplateIdentifiers(templateIdentifiers)
  method.setVirtual(1)
  method.setPurpose([STUB, CARTILAGE, IOR])
  return method

def p_return_parameter_specifier_1(t):
  '''return_parameter_specifier : VOID'''
  pass

def p_return_parameter_specifier_2(t):
  '''return_parameter_specifier : type_specifier'''
  parameter = ASE.Compiler.SIDL.Parameter.Parameter()
  parameter.setIdentifier('_return_value')
  parameter.setType(t[1])
  t[1].setParent(parameter)
  parameter.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
  parameter.setRequired(1)
  t[0] = parameter

def p_return_parameter_specifier_3(t):
  '''return_parameter_specifier : type_specifier COLON ID'''
  parameter = ASE.Compiler.SIDL.Parameter.Parameter()
  parameter.setIdentifier(t[3])
  parameter.setType(t[1])
  t[1].setParent(parameter)
  parameter.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
  parameter.setRequired(1)
  t[0] = parameter

def p_throws_specifier(t):
  'throws_specifier : THROWS scoped_identifier_list'
  t[0] = t[2]

def p_throws_specifier_empty(t):
  'throws_specifier : empty'
  t[0] = None

# parameter-list
def p_parameter_list_1(t): 
  'parameter_list : parameter COMMA parameter_list'
  t[0] = [t[1]]+t[3]

def p_parameter_list_2(t): 
  'parameter_list : parameter'
  t[0] = [t[1]]

def p_parameter_list_empty(t): 
  'parameter_list : empty'
  t[0] = []

def p_parameter(t):
  'parameter : access_specifier type_specifier ID'
  parameter = ASE.Compiler.SIDL.Parameter.Parameter()
  parameter.setIdentifier(t[3])
  parameter.setType(t[2])
  t[2].setParent(parameter)
  parameter.setAccessMode(int(t[1]))
  parameter.setRequired(1)
  t[0] = parameter

def p_access_specifier(t):
  '''access_specifier : IN
                      | INOUT
                      | OUT'''
  if t[1] == 'in':
    t[0] = ASE.Compiler.ParameterAccessMode.IN
  elif t[1] == 'inout':
    t[0] = ASE.Compiler.ParameterAccessMode.INOUT
  elif t[1] == 'out':
    t[0] = ASE.Compiler.ParameterAccessMode.OUT
  else:
    raise RuntimeError('Invalid access specifier: '+t[1])

# type-list:
def p_type_list_1(t):
  'type_list : type_list COMMA type_specifier'
  t[0] = t[1]+[t[3]]

def p_type_list_2(t):
  'type_list : type_specifier'
  t[0] = [t[1]]

# type-specifier:
def p_type_specifier_1(t):
  '''type_specifier : BOOL
                    | CHAR
                    | DCOMPLEX
                    | DOUBLE
                    | FCOMPLEX
                    | FLOAT
                    | INT
                    | LONG
                    | OPAQUE
                    | STRING'''
  if t[1] in baseTypes:
    type = baseTypes[t[1]]
  else:
    type = ASE.Compiler.SIDL.Type.Type()
    type.setIdentifier(t[1])
    type.setBaseType(1)
    baseTypes[t[1]] = type
  t[0] = type

def p_type_specifier_2(t):
  '''type_specifier : scoped_identifier'''
  type = ASE.Compiler.SIDL.Type.Type()
  type.setIdentifier(t[1])
  t[0] = type

def p_type_specifier_3(t):
  '''type_specifier : array_specifier'''
  t[0] = t[1]

# array-specifier
def p_array_specifier_1(t):
  'array_specifier : ARRAY LT type_specifier COMMA ICONST GT'
  array = ASE.Compiler.SIDL.Array.Array()
  array.setIdentifier('array')
  array.setType(t[3])
  t[3].setParent(array)
  array.setDimension(int(t[5]))
  t[0] = array

def p_array_specifier_2(t):
  'array_specifier : ARRAY LT type_specifier GT'
  array = ASE.Compiler.SIDL.Array.Array()
  array.setIdentifier('array')
  array.setType(t[3])
  t[3].setParent(array)
  t[0] = array

# scoped-identifier-list
def p_scoped_identifier_list_1(t):
  'scoped_identifier_list : scoped_identifier_list COMMA scoped_identifier'
  t[0] = t[1]+[t[3]]

def p_scoped_identifier_list_2(t):
  'scoped_identifier_list : scoped_identifier'
  t[0] = [t[1]]

def p_scoped_identifier(t):
  '''scoped_identifier : SCOPEDID
                       | ID'''
  t[0] = t[1]

def p_optional_semi(t):
  '''optional_semi : SEMICOLON 
                   | empty''' 
  pass

def p_comment_list_1(t):
  'comment_list : comment_list comment'
  t[0] = t[1] + [t[2]]

def p_comment_list_2(t):
  'comment_list : comment'
  t[0] = [t[1]]

def p_comment(t):
  'comment : COMMENT'
  comment = ASE.Compiler.SIDL.Comment.Comment()
  comment.setComment(removeCommentMarkers(t[1]))
  t[0] = comment

def p_empty(t):
  'empty : '
  pass

# Parse error
def p_error(t):
  print "Error: %s " % t

# Driver
def setup(debug = 0):
  global parser

  parser = yacc.yacc(debug = debug)
  return parser

def removeCommentMarkers(comment):
  if comment.startswith('/*'):
    if not comment.endswith('*/'):
      raise RuntimeError('Invalid comment:\n'+comment)
    newComment = comment[2:-2]
  if comment.startswith('//'):
    newComment = comment[2:]
  newComment.strip()
  return newComment

if __name__ == '__main__':
  import ASE.Compiler.SIDL.Printer

  #import profile
  # Build the grammar
  #profile.run("yacc.yacc()")

  setup()

  for i in range(1, len(sys.argv)):
    print "About to parse %s" % sys.argv[i]
    f = open(sys.argv[i],"r")
    s = f.read()
    f.close()
    # print "Contents of %s: %s" % (sys.argv[i], s)
    ast     = parser.parse(s)
    printer = ASE.Compiler.SIDL.Printer.Printer()
    ast.accept(printer)
